import asyncio
from loguru import logger

from pyjabber.network.ConnectionManager import ConnectionManager
from pyjabber.utils import Singleton


class QueueMessage(metaclass=Singleton):
    def __init__(self, connection_manager):
        self._queue = []
        self._loop = asyncio.get_event_loop()
        self._chrono_task = asyncio.Event()

        self._connection_manager: ConnectionManager = connection_manager

    def enqueue(self, host, element):
        self._queue.append((host, element))
        self._loop.create_task(self.chrono())

    async def chrono(self):
        while len(self._queue) > 0:
            await asyncio.sleep(5)
            for host, element in self._queue:
                buffer = self._connection_manager.get_server_buffer(host)
                if buffer:
                    buffer[-1].write(element)
                    logger.debug(f"Routed message to {host}")
                    self._queue.remove((host, element))



import xml.etree.ElementTree as ET
from contextlib import closing

from pyjabber.db.database import connection
from pyjabber.plugins.PluginInterface import Plugin
from pyjabber.stanzas.error import StanzaError as SE
from pyjabber.stanzas.IQ import IQ


class Roster(Plugin):
    def __init__(self, jid: str, db_connection_factory=None) -> None:
        self._jid = jid
        self._handlers = {
            "get": self.handle_get,
            "set": self.handle_set,
            "result": self.handle_result
        }
        self._ns = {
            "ns": "jabber:iq:roster",
            "query": "{jabber:iq:roster}query",
            "item": "{jabber:iq:roster}item"
        }
        self._db_connection_factory = db_connection_factory or connection

    def feed(self, element: ET.Element):
        if len(element) != 1:
            return SE.invalid_xml()

        return self._handlers[element.attrib["type"]](element)

    ############################################################
    ############################################################

    def handle_get(self, element: ET.Element):
        jid = self._jid.split("/")[0]
        try:
            with closing(self._db_connection_factory()) as con:
                res = con.execute("SELECT * FROM roster WHERE jid = ?", (jid,))
                roster = res.fetchall()

            iq_res = IQ(
                type=IQ.TYPE.RESULT.value,
                id=element.attrib["id"]
            )

            query = ET.SubElement(
                iq_res,
                "query",
                attrib={"xmlns": self._ns["ns"]}
            )

            for item in roster:
                query.append(ET.fromstring(item[-1]))

            return ET.tostring(iq_res)

        except BaseException:
            raise Exception()

    def handle_set(self, element: ET.Element):
        query = element.find(self._ns["query"])
        jid = self._jid.split("/")[0]

        if query is None:
            raise Exception()

        new_item = query.findall(self._ns["item"])

        if len(new_item) != 1:
            raise Exception()

        new_item = new_item[0]
        if "subscription" in new_item.attrib.keys():
            remove = new_item.attrib["subscription"] == "remove"

        with closing(self._db_connection_factory()) as con:
            res = con.execute("SELECT * FROM roster WHERE jid = ?", (jid,))
            res = res.fetchall()

            roster = res
            roster = [ET.fromstring(r[-1]) for r in roster]
            match_item = [i for i in roster if i.attrib["jid"] == new_item.attrib["jid"]]

            if match_item:
                # Delete roster item
                if remove:
                    con.execute("""
                                DELETE FROM roster
                                WHERE jid = ? AND rosterItem = ?
                                """,
                                (jid,
                                 ET.tostring(match_item[0]).decode()))
                    con.commit()

                else:
                    con.execute("""
                                UPDATE roster
                                SET rosterItem = ?
                                WHERE jid = ? AND rosterItem = ?
                                """,
                                (ET.tostring(new_item).decode(),
                                 jid,
                                 ET.tostring(match_item[0])))
                    con.commit()

            else:
                if not remove:
                    con.execute(
                        "INSERT INTO roster(jid, rosterItem) VALUES (?, ?)",
                        (jid,
                         ET.tostring(new_item).decode()))
                    con.commit()

            res = ET.Element(
                "iq",
                attrib={
                    "id": element.attrib["id"],
                    "type": "result"
                }
            )

            return ET.tostring(res)

    def handle_result(self, element: ET.Element):
        # It's safe to ignore this stanza
        return



import pytest
from unittest.mock import Mock, MagicMock, patch
from xml.etree import ElementTree as ET
from pyjabber.utils import ClarkNotation as CN
from pyjabber.stream.StreamHandler import Signal, StreamHandler
from pyjabber.network.ConnectionManager import ConnectionManager
from pyjabber.network.XMLParser import XMLParser


def test_initialization():
    buffer = Mock()
    starttls = Mock()
    connection_manager = Mock()
    queue_message = Mock()
    host = Mock()

    handler = XMLParser(host, buffer, starttls, connection_manager, queue_message)

    assert handler._state == XMLParser.StreamState.CONNECTED
    assert handler._buffer == buffer
    assert handler._stanzaHandler is None
    assert handler._streamHandler is not None
    assert handler._stack == []
    assert handler._connection_manager == connection_manager
    assert handler._queue_message == queue_message


def test_buffer_property():
    buffer = Mock()
    starttls = Mock()
    connection_manager = Mock()
    queue_message = Mock()
    host  = Mock()

    handler = XMLParser(host, buffer, starttls, connection_manager, queue_message)

    new_buffer = Mock()
    handler.buffer = new_buffer

    assert handler.buffer == new_buffer
    assert handler._streamHandler.buffer == new_buffer


def test_start_element_ns_stream():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    queue_message = Mock()
    host = Mock()

    handler = XMLParser(host, buffer, starttls, connection_manager, queue_message)

    attrs = {("http://etherx.jabber.org/streams", "version"): "1.0"}
    handler.startElementNS(("http://etherx.jabber.org/streams", "stream"), "stream", attrs)

    assert buffer.write.call_count == 2
    assert len(handler._stack) == 1
    assert handler._stack[0].tag == CN.clarkFromTuple(("http://etherx.jabber.org/streams", "stream"))


def test_start_element_ns_normal_element():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    queue_message = Mock()
    host = Mock()

    handler = XMLParser(host, buffer, starttls, connection_manager, queue_message)

    attrs = {("namespace", "attr"): "value"}
    handler._stack.append(ET.Element("dummy"))

    handler.startElementNS(("namespace", "element"), "element", attrs)
    assert len(handler._stack) == 2
    assert handler._stack[-1].tag == CN.clarkFromTuple(("namespace", "element"))
    assert handler._stack[-1].attrib == {CN.clarkFromTuple(("namespace", "attr")): "value"}


def test_start_element_ns_invalid():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    queue_message = Mock()
    host = Mock()

    handler = XMLParser(host, buffer, starttls, connection_manager, queue_message)

    with pytest.raises(Exception):
        handler.startElementNS(("invalid", "element"), "element", {})


def test_end_element_ns_stream():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    queue_message = Mock()
    host = Mock()

    handler = XMLParser(host, buffer, starttls, connection_manager, queue_message)
    handler._stack.append(ET.Element("{http://etherx.jabber.org/streams}stream"))

    handler.endElementNS(("http://etherx.jabber.org/streams", "stream"), "stream")

    buffer.write.assert_called_once_with(b'</stream:stream>')
    assert len(handler._stack) == 0


def test_end_element_ns_normal_element():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    queue_message = Mock()
    host = Mock()
    handler = XMLParser(host, buffer, starttls, connection_manager, queue_message)

    parent = ET.Element(CN.clarkFromTuple(("namespace", "parent")))
    child = ET.Element(CN.clarkFromTuple(("namespace", "child")))

    handler._stack.append(parent)
    handler._stack.append(child)

    handler.endElementNS(("namespace", "child"), "child")

    assert len(handler._stack) == 1
    assert len(handler._stack[0]) == 1
    assert handler._stack[0][0].tag == CN.clarkFromTuple(("namespace", "child"))

def test_end_element_ns_invalid_stack():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    queue_message = Mock()
    host = Mock()
    handler = XMLParser(host, buffer, starttls, connection_manager, queue_message)

    with pytest.raises(Exception):
        handler.endElementNS(("namespace", "element"), "element")


def test_end_element_ns_mismatched_tag():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    queue_message = Mock()
    host = Mock()
    handler = XMLParser(host, buffer, starttls, connection_manager, queue_message)
    handler._stack.append(ET.Element("different"))

    with pytest.raises(Exception):
        handler.endElementNS(("namespace", "element"), "element")


@patch('pyjabber.stream.StanzaHandler.StanzaHandler')
@patch('pyjabber.stream.StreamHandler.Signal')
@patch('pyjabber.stream.StreamHandler.StreamHandler.handle_open_stream')
def test_end_element_ns_stream_handling(mock_handle_open_stream, mock_signal, mock_stanza_handler):
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    queue_message = Mock()
    host = Mock()
    handler = XMLParser(host, buffer, starttls, connection_manager, queue_message)
    handler._stack.append(ET.Element("{http://etherx.jabber.org/streams}stream"))

    elem = ET.Element(CN.clarkFromTuple(("namespace", "dummy")))
    handler._stack.append(elem)

    mock_signal.RESET = Signal.RESET
    mock_signal.DONE = Signal.DONE
    mock_handle_open_stream.return_value = Signal.DONE

    mock_stanza = mock_stanza_handler.return_value
    mock_stanza.feed = MagicMock()

    handler.endElementNS(("namespace", "dummy"), "dummy")

    assert handler._state == XMLParser.StreamState.READY


def test_characters():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    queue_message = Mock()
    host = Mock()
    handler = XMLParser(host, buffer, starttls, connection_manager, queue_message)
    parent = ET.Element("parent")
    child = ET.SubElement(parent, "child")
    handler._stack.append(parent)

    # Verificación cuando el elemento tiene hijos
    handler.characters("content")
    assert child.tail == "content"

    # Verificación cuando el elemento no tiene hijos
    handler._stack[-1] = ET.Element("parent_no_children")
    handler.characters("more content")
    assert handler._stack[-1].text == "more content"
    handler.characters(" additional content")
    assert handler._stack[-1].text == "more content additional content"

if __name__ == "__main__":
    pytest.main()



from asyncio import BaseProtocol
from enum import Enum
from xml.etree import ElementTree as ET
from xml.sax import ContentHandler

from loguru import logger

from pyjabber.stream import Stream
from pyjabber.stream.StanzaHandler import StanzaHandler
from pyjabber.stream.StreamHandler import Signal, StreamHandler
from pyjabber.utils import ClarkNotation as CN


class XMLParser(ContentHandler):
    """
        Manages the stream data and process the XML objects.
        Inheriting from sax.ContentHandler

        :param host: Host of the running server
        :param buffer: Transport instance of the connected client. Used to send replays
        :param starttls: Coroutine launched when server and client start the connection upgrade process to TLS
        :param connection_manager: Global instance of the Connection Manager class (Singleton)
        :param queue_message: Global instance of Queue Message class (Singleton)
    """

    def __init__(self, host, buffer, starttls, connection_manager, queue_message):
        super().__init__()

        self._host = host
        self._buffer = buffer
        self._connection_manager = connection_manager
        self._queue_message = queue_message

        self._state = self.StreamState.CONNECTED
        self._stanzaHandler = None
        self._stack = []
        self._streamHandler = StreamHandler(
            self._host, self._buffer, starttls, connection_manager)

    class StreamState(Enum):
        """
        Stream connection states.
        """
        CONNECTED = 0
        READY = 1

    @property
    def buffer(self) -> BaseProtocol:
        return self._buffer

    @buffer.setter
    def buffer(self, value: BaseProtocol):
        self._buffer = value
        self._streamHandler.buffer = value

    def startElementNS(self, name, qname, attrs):
        logger.debug(f"Start element from <{hex(id(self._buffer))}>: {name}")

        if self._stack:  # "<stream:stream>" tag already present in the data stack
            elem = ET.Element(
                CN.clarkFromTuple(name),
                attrib={
                    CN.clarkFromTuple(key): item for key,
                    item in dict(attrs).items()})
            self._stack.append(elem)

        elif name[1] == "stream" and name[0] == "http://etherx.jabber.org/streams":
            self._buffer.write(Stream.responseStream(attrs))

            elem = ET.Element(
                CN.clarkFromTuple(name),
                attrib={
                    CN.clarkFromTuple(key): item for key,
                    item in dict(attrs).items()})

            self._stack.append(elem)
            self._streamHandler.handle_open_stream()

        else:
            raise Exception()

    def endElementNS(self, name, qname):
        logger.debug(f"End element NS: {qname} : {name}")

        if "stream" in name:
            self._buffer.write(b'</stream:stream>')
            self._stack.clear()
            return

        if not self._stack:
            raise Exception()

        elem = self._stack.pop()

        if elem.tag != CN.clarkFromTuple(name):
            # INVALID STANZA/MESSAGE
            raise Exception()

        if self._stack[-1].tag != '{http://etherx.jabber.org/streams}stream':
            self._stack[-1].append(elem)

        else:
            if self._state == self.StreamState.READY:  # Ready to process stanzas
                self._stanzaHandler.feed(elem)
            else:
                signal = self._streamHandler.handle_open_stream(elem)
                if signal == Signal.RESET and "stream" in self._stack[-1].tag:
                    self._stack.clear()
                elif signal == Signal.DONE:
                    self._stanzaHandler = StanzaHandler(
                        self._host, self._buffer, self._connection_manager, self._queue_message)
                    self._state = self.StreamState.READY

    def characters(self, content: str) -> None:
        if not self._stack:
            raise Exception()

        elem = self._stack[-1]
        if len(elem) != 0:
            child = elem[-1]
            child.tail = (child.tail or '') + content

        else:
            elem.text = (elem.text or '') + content



import pytest
import asyncio
from unittest.mock import MagicMock, patch
from pyjabber.network.ConnectionManager import ConnectionManager
from pyjabber.stream.QueueMessage import QueueMessage


@pytest.fixture
def setup_queue_message(event_loop):
    connection_manager = MagicMock(ConnectionManager)
    queue_message = QueueMessage(connection_manager)
    return queue_message, connection_manager

@pytest.fixture
def event_loop():
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    yield loop
    loop.close()

@pytest.mark.asyncio
async def test_enqueue_and_chrono(setup_queue_message):
    queue_message, connection_manager = setup_queue_message
    host = 'test_host'
    element = b'<message>Test</message>'

    buffer = [MagicMock()]
    connection_manager.get_server_buffer.return_value = buffer

    queue_message.enqueue(host, element)

    await asyncio.sleep(6)  # Esperar a que se ejecute la tarea chrono

    connection_manager.get_server_buffer.assert_called_with(host)
    buffer[-1].write.assert_called_with(element)
    assert len(queue_message._queue) == 0

def test_initialization(setup_queue_message):
    queue_message, connection_manager = setup_queue_message
    assert isinstance(queue_message._queue, list)

# @pytest.mark.asyncio
# async def test_chrono_no_buffer(setup_queue_message):
#     queue_message, connection_manager = setup_queue_message
#     host = 'test_host'
#     element = b'<message>Test</message>'
#
#     connection_manager.get_server_buffer.return_value = None
#
#     queue_message.enqueue(host, element)
#
#     await asyncio.sleep(6)  # Esperar a que se ejecute la tarea chrono
#
#     connection_manager.get_server_buffer.assert_called_with(host)
#     assert len(queue_message._queue) == 1  # Elemento sigue en la cola

if __name__ == "__main__":
    pytest.main()



import asyncio
import os

from aiohttp import web
from loguru import logger

from pyjabber.webpage.api import api

FILE_PATH = os.path.dirname(os.path.abspath(__file__))


async def admin_instance():
    app = web.Application()
    app.router.add_get('/', getIndex)
    app.router.add_get('/api/users', api.handleUser)
    app.router.add_get('/api/roster/{id}', api.handleRoster)
    app.router.add_post('/api/createuser', api.handleRegister)
    app.router.add_delete('/api/users/{id}', api.handleDelete)
    app.router.add_static('/static', FILE_PATH + '/build/static')
    app.router.add_static('/', FILE_PATH + '/build/')

    runner = web.AppRunner(app)
    await runner.setup()
    site = web.TCPSite(runner, 'localhost', 9090)
    await site.start()

    logger.info("Serving admin webpage on http://localhost:9090")
    while True:
        await asyncio.sleep(3600)  # Keep alive the server


async def getIndex(request):
    return web.FileResponse(FILE_PATH + '/build/index.html')



import os
import pickle
import re
import socket
import xml.etree.ElementTree as ET

import xmlschema

from pyjabber.features.presence.PresenceFeature import Presence
from pyjabber.plugins.PluginManager import PluginManager
from pyjabber.stanzas.error import StanzaError as SE
from pyjabber.utils import ClarkNotation as CN

FILE_PATH = os.path.dirname(os.path.abspath(__file__))


class StanzaHandler:
    def __init__(self, host, buffer, connection_manager, queue_message) -> None:
        self._host = host
        self._buffer = buffer
        self._connections = connection_manager
        self._queue_message = queue_message

        self._peername = buffer.get_extra_info('peername')
        self._jid = self._connections.get_jid(self._peername)

        self._pluginManager = PluginManager(self._jid)
        self._presenceManager = Presence(self._jid, self._connections)

        self._functions = {
            "{jabber:client}iq": self.handle_iq,
            "{jabber:client}message": self.handle_msg,
            "{jabber:client}presence": self.handle_pre
        }

        with open(FILE_PATH + "/schemas/schemas.pkl", "rb") as schemasDump:
            self._schemas = pickle.load(schemasDump)

    def feed(self, element: ET.Element):
        try:
            schema: xmlschema.XMLSchema = self._schemas[CN.deglose(element.tag)[0]]
            if schema.is_valid(ET.tostring(element)) is False:
                self._buffer.write(SE.bad_request())
                return
        except KeyError:
            self._buffer.write(SE.feature_not_implemented())
            return

        try:
            self._functions[element.tag](element)
        except KeyError:
            raise Exception()

    ############################################################
    ############################################################

    def handle_iq(self, element: ET.Element):
        """
            Process the iq stanza with the PluginManager (PM) class
            If the feature/XEP requested is not available, the PM instance
            will send a

            :param element: The stanza in the ElementTree format
        """
        res = self._pluginManager.feed(element)
        if res:
            self._buffer.write(res)

    def handle_msg(self, element: ET.Element):
        """
            Router the message to the client

            If the destination client is a user of a remote server, it will queue the message into the QueueMessage
            object and try to connect to the remote server

            :param element: the message in the ElementTree format
        """
        bare_jid = element.attrib["to"].split("/")[0]

        if re.match(fr'^[a-zA-Z0-9._%+-]+@{re.escape(self._host)}$', bare_jid):
            for buffer in self._connections.get_buffer(bare_jid):
                buffer[-1].write(ET.tostring(element))

        else:
            server_buffer = self._connections.get_server_buffer(bare_jid)
            if server_buffer:
                server_buffer[-1].write(ET.tostring(element))

            else:
                server_host = element.attrib["to"].split("@")[-1]
                self._queue_message.enqueue(server_host, ET.tostring(element))

    def handle_pre(self, element: ET.Element):
        res = self._presenceManager.feed(element, self._jid)
        if res:
            self._buffer.write(res)



import xml.etree.ElementTree as ET
from uuid import uuid4


def create_roster_entry(jid, to, roster_manager):
    iq = ET.Element(
        "iq", attrib={"from": jid, "id": str(uuid4()), "type": "set"}
    )
    query = ET.Element("{jabber:iq:roster}query")
    item = ET.Element(
        "{jabber:iq:roster}item",
        attrib={
            "jid": to,
            "subscription": "none"})
    query.append(item)
    iq.append(query)

    return roster_manager.feed(iq)



import pytest
import sqlite3
import base64
import hashlib
from xml.etree import ElementTree as ET
from unittest.mock import MagicMock, patch

from pyjabber.features.SASLFeature import SASL, SASLFeature, Signal, mechanismEnum
from pyjabber.stanzas.error import StanzaError as SE
from pyjabber.utils import ClarkNotation as CN


@pytest.fixture
def setup_database():
    con = sqlite3.connect(':memory:')
    cur = con.cursor()
    cur.execute('''
        CREATE TABLE credentials (
            jid VARCHAR(255) NOT NULL PRIMARY KEY,
            hash_pwd VARCHAR(255) NOT NULL
        )
    ''')
    cur.execute('''
        INSERT INTO credentials (jid, hash_pwd) VALUES
        ('username', ?)
    ''', (hashlib.sha256(b'password').hexdigest(),))
    con.commit()
    yield con
    con.close()


@pytest.fixture
def db_connection_factory(setup_database):
    def factory():
        return setup_database

    return factory


@patch('pyjabber.network.ConnectionManager.ConnectionManager')
def test_handle_auth_success(MockConnectionsManager, db_connection_factory):
    sasl = SASL(MockConnectionsManager())
    # sasl._connections = MockConnectionsManager()
    sasl._db_connection_factory = db_connection_factory
    element = ET.Element("{urn:ietf:params:xml:ns:xmpp-sasl}auth")
    auth_text = base64.b64encode(b'\x00username\x00password').decode('ascii')
    element.text = auth_text

    result = sasl.handleAuth(element)

    assert result == (Signal.RESET, b"<success xmlns='urn:ietf:params:xml:ns:xmpp-sasl'/>")


@patch('pyjabber.network.ConnectionManager.ConnectionManager')
def test_handle_auth_failure(MockConnectionsManager, db_connection_factory):
    sasl = SASL(MockConnectionsManager())
    # sasl._connections = MockConnectionsManager()
    sasl._db_connection_factory = db_connection_factory
    element = ET.Element("{urn:ietf:params:xml:ns:xmpp-sasl}auth")
    auth_text = base64.b64encode(b'\x00username\x00wrongpassword').decode('ascii')
    element.text = auth_text

    result = sasl.handleAuth(element)

    assert result == SE.not_authorized()


@patch('pyjabber.network.ConnectionManager.ConnectionManager')
def test_handle_iq_register_conflict(MockConnectionsManager, db_connection_factory):
    sasl = SASL(MockConnectionsManager(), db_connection_factory)
    element = ET.Element("iq", attrib={"type": "set", "id": "123"})
    query = ET.SubElement(element, "{jabber:iq:register}query")
    username = ET.SubElement(query, "{jabber:iq:register}username")
    username.text = "username"
    password = ET.SubElement(query, "{jabber:iq:register}password")
    password.text = "password"

    result = sasl.handleIQ(element)

    expected_result = (Signal.RESET, SE.conflict_error("123"))
    assert result == expected_result


@patch('pyjabber.network.ConnectionManager.ConnectionManager')
def test_handle_iq_register_success(MockConnectionsManager, db_connection_factory):
    sasl = SASL(MockConnectionsManager(), db_connection_factory)
    element = ET.Element("iq", attrib={"type": "set", "id": "123"})
    query = ET.SubElement(element, "{jabber:iq:register}query")
    username = ET.SubElement(query, "{jabber:iq:register}username")
    username.text = "new_user"
    password = ET.SubElement(query, "{jabber:iq:register}password")
    password.text = "password"

    result = sasl.handleIQ(element)

    expected_result = sasl.iq_register_result("123")
    assert result == (Signal.RESET, expected_result)


def test_sasl_feature():
    mechanisms = [mechanismEnum.PLAIN, mechanismEnum.SCRAM_SHA_1]
    feature_element = SASLFeature(mechanisms)

    assert feature_element.tag == "mechanisms"
    assert feature_element.attrib == {"xmlns": "urn:ietf:params:xml:ns:xmpp-sasl"}
    assert len(feature_element) == 2
    assert feature_element[0].text == mechanismEnum.PLAIN.value
    assert feature_element[1].text == mechanismEnum.SCRAM_SHA_1.value


if __name__ == "__main__":
    pytest.main()



import base64

from typing import Union
from xml.etree import ElementTree as ET

from pyjabber.features.StartTLSFeature import StartTLSFeature
from pyjabber.features.SASLFeature import SASLFeature, mechanismEnum
from pyjabber.stream.StreamHandler import StreamHandler, Signal, Stage


class StreamServerIncomingHandler(StreamHandler):
    STARTTLS = "{urn:ietf:params:xml:ns:xmpp-tls}starttls"
    AUTH = "{urn:ietf:params:xml:ns:xmpp-sasl}auth"

    def __init__(self, host, buffer, starttls, connection_manager) -> None:
        super().__init__(host, buffer, starttls, connection_manager)

    def handle_open_stream(self, elem: ET.Element = None) -> Union[Signal, None]:
        if elem is None:
            if self._stage == Stage.CONNECTED:
                self._streamFeature.reset()
                self._streamFeature.register(StartTLSFeature())
                self._buffer.write(self._streamFeature.to_bytes())
                self._stage = Stage.OPENED
                return

            elif self._stage == Stage.SSL:
                self._streamFeature.reset()
                self._streamFeature.register(SASLFeature(mechanismList=[mechanismEnum.EXTERNAL]))
                self._buffer.write(self._streamFeature.to_bytes())
                self._stage = Stage.SASL
                return

            elif self._stage == Stage.AUTH:
                self._buffer.write(b"<features xmlns='http://etherx.jabber.org/streams'/>")
                self._stage = Stage.READY
                return Signal.DONE

            else:
                raise Exception()

        elif self._stage == Stage.OPENED and elem.tag == self.STARTTLS:
            self._buffer.write(StartTLSFeature().proceed_response())
            self._starttls()
            self._stage = Stage.SSL
            return Signal.RESET

        elif self._stage == Stage.SASL and elem.tag == self.AUTH:
            if "mechanism" in elem.attrib.keys() and elem.attrib["mechanism"] == "EXTERNAL":
                if elem.text is None:
                    raise Exception()
                elif elem.text == "=":
                    pass
                else:
                    host = base64.b64decode(elem.text).decode()
                    self._buffer.write(b"<success xmlns='urn:ietf:params:xml:ns:xmpp-sasl'/>")
                    self._stage = Stage.AUTH
                    return Signal.RESET

        else:
            self._buffer.write(b"<features xmlns='http://etherx.jabber.org/streams'/>")



import asyncio
import os
import ssl

from loguru import logger
from xml import sax

from pyjabber.network.ConnectionManager import ConnectionManager
from pyjabber.network.StreamAlivenessMonitor import StreamAlivenessMonitor
from pyjabber.network.XMLParser import XMLParser

FILE_AUTH = os.path.dirname(os.path.abspath(__file__))


class XMLProtocol(asyncio.Protocol):
    """
    Protocol to manage the network connection between nodes in the XMPP network. Handles the transport layer.

    :param namespace: namespace of the XML tags (jabber:client or jabber:server)
    :param host: Host for connections
    :param connection_timeout: Max time without any response from a client. After that, the server will terminate the connection
    :param connection_manager: Global instance of Connection Manager (Singleton)
    :param cert_path: Path to custom domain certs. By default, the server generates its own certificates for hostname
    :param queue_message: Global instance of Queue Message class (Singleton)
    :param enable_tls1_3: Boolean. Enables the use of TLSv1.3 in the STARTTLS process
    """

    def __init__(
            self,
            namespace,
            host,
            connection_timeout,
            connection_manager,
            cert_path,
            queue_message,
            enable_tls1_3=False):

        self._xmlns = namespace
        self._host = host
        self._connection_timeout = connection_timeout
        self._connection_manager: ConnectionManager = connection_manager
        self._cert_path = cert_path
        self._queue_message = queue_message
        self._enable_tls1_3 = enable_tls1_3

        self._transport = None
        self._xml_parser = None
        self._timeout_monitor = None


    def connection_made(self, transport):
        """
        Called when a client or another server opens a TCP connection to the server

        :param transport: The transport object for the connection
        :type transport: asyncio.Transport
        """
        if transport:
            self._transport = transport

            self._xml_parser = sax.make_parser()
            self._xml_parser.setFeature(sax.handler.feature_namespaces, True)
            self._xml_parser.setFeature(sax.handler.feature_external_ges, False)
            self._xml_parser.setContentHandler(
                XMLParser(
                    self._host,
                    self._transport,
                    self.task_tls,
                    self._connection_manager,
                    self._queue_message
                )
            )

            if self._connection_timeout:
                self._timeout_monitor = StreamAlivenessMonitor(
                    timeout=self._connection_timeout,
                    callback=self.connection_timeout
                )

            self._connection_manager.connection(self._transport.get_extra_info('peername'))

            logger.info(f"Connection from {self._transport.get_extra_info('peername')}")
        else:
            logger.error("Invalid transport")

    def connection_lost(self, exc):
        """
        Called when a client or another server closes a TCP connection to the server

        :param exc: Exception that caused the connection to close
        :type exc: Exception
        """
        logger.info(f"Connection lost from {self._transport.get_extra_info('peername')}: Reason {exc}")

        self._transport = None
        self._xml_parser = None

    def data_received(self, data):
        """
        Called when data is received from the client or another server

        :param data: Chunk of data received
        :type data: Byte array
        """
        try:
            logger.debug(f"Data received from <{hex(id(self._transport))}>: {data.decode()}")
        except:
            logger.debug(f"Binary data recived")

        if self._timeout_monitor:
            self._timeout_monitor.reset()

        '''
        Some XMPP clients/libraries send the XML header
        with the stream stanza all together.
        This can lead sometimes in a exception for the parser.
        An XML header gives no usefull information, as the RFC6120
        takes the vesion 1.0 as standar, so we can remove it safely.
        If I try to remove the header inside the except block, the parser
        keeps refusing it. I'm forced to do always before the feed.
        I probably should change the parser
        '''
        data = data.replace(b"<?xml version=\'1.0\'?>", b"")
        data = data.replace(b"<?xml version=\"1.0\"?>", b"")

        self._xml_parser.feed(data)

    def eof_received(self):
        """
        Called when the client or another server sends an EOF
        """
        peer = self._transport.get_extra_info('peername')

        logger.debug(f"EOF received from {peer}")

        self._connection_manager.disconnection(peer)

    def connection_timeout(self):
        """
        Called when the stream is not responding for a long tikem
        """
        peer = self._transport.get_extra_info('peername')
        logger.debug(f"Connection timeout from {peer}")

        self._transport.write("<connection-timeout/>".encode())
        self._transport.close(peer)

        self._connection_manager.disconnection(peer)

        self._transport = None
        self._xml_parser = None

    ###########################################################################
    ###########################################################################
    ###########################################################################

    def task_tls(self):
        """
            Sync function to call the STARTTLS coroutine
        """
        loop = asyncio.get_running_loop()
        asyncio.ensure_future(self.enable_tls(loop), loop=loop)

    async def enable_tls(self, loop):
        """
            Coroutine to upgrade the connection to TLS
            It swaps the transport for the XMLProtocol, and XMLParser
            :param loop: Running asyncio loop
        """
        parser = self._xml_parser.getContentHandler()

        ssl_context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
        if not self._enable_tls1_3:
            ssl_context.options |= ssl.OP_NO_TLSv1_3

        if self._cert_path:
            ssl_context.load_cert_chain(
                certfile=os.path.join(self._cert_path, f"{self._host}_cert.pem"),
                keyfile=os.path.join(self._cert_path, "certs", f"{self._host}_key.pem"),
            )
        else:
            ssl_context.load_cert_chain(
                certfile=os.path.join(FILE_AUTH, "certs", f"{self._host}_cert.pem"),
                keyfile=os.path.join(FILE_AUTH, "certs", f"{self._host}_key.pem"),
            )

        self._transport = await loop.start_tls(
            transport=self._transport,
            protocol=self,
            sslcontext=ssl_context,
            server_side=True)

        parser.buffer = self._transport
        logger.debug(f"Done TLS for <{hex(id(self))}>")



import os
import xml.etree.ElementTree as ET

from pyjabber.features.presence.PresenceFeature import Presence
from pyjabber.stanzas.error import StanzaError as SE

FILE_PATH = os.path.dirname(os.path.abspath(__file__))


class StanzaServerIncomingHandler:
    def __init__(self, buffer, connection_manager) -> None:
        self._buffer = buffer
        self._connection_manager = connection_manager
        self._peername = buffer.get_extra_info('peername')
        self._host = self._connection_manager.get_server_host(self._peername)
        self._presenceManager = Presence(self._host)

        self._functions = {
            "{jabber:client}iq": self.handle_iq,
            "{jabber:client}message": self.handle_msg,
            "{jabber:client}presence": self.handle_pre
        }

    def feed(self, element: ET.Element):
        try:
            self._functions[element.tag](element)
        except KeyError:
            self._buffer.write(SE.bad_request())

    ############################################################
    ############################################################

    def handle_iq(self, element: ET.Element):
        return

    def handle_msg(self, element: ET.Element):
        receiver_buffer = self._connection_manager.get_buffer(element.attrib["to"])

        for buffer in receiver_buffer:
            buffer[-1].write(ET.tostring(element))

    def handle_pre(self, element: ET.Element):
        return



import os
import pytest
from unittest.mock import MagicMock, patch
from pyjabber.network.server.incoming.XMLServerIncomingProtocol import XMLServerIncomingProtocol
from pyjabber.network.server.incoming.XMLServerIncomingParser import XMLServerIncomingParser
from pyjabber.network.StreamAlivenessMonitor import StreamAlivenessMonitor


@pytest.fixture
def setup_protocol():
    namespace = 'namespace'
    host = 'host'
    connection_timeout = 10
    connection_manager = MagicMock()
    cert_path = 'cert_path'
    queue_message = MagicMock()
    protocol = XMLServerIncomingProtocol(
        namespace,
        host,
        connection_timeout,
        connection_manager,
        cert_path,
        queue_message
    )
    return protocol, connection_manager


def test_initialization(setup_protocol):
    protocol, connection_manager = setup_protocol
    assert protocol._host == 'host'
    assert protocol._connection_timeout == 10
    assert protocol._connection_manager == connection_manager
    assert protocol._queue_message is not None


@patch('xml.sax.make_parser', return_value=MagicMock())
def test_connection_made_with_transport(mock_make_parser, setup_protocol):
    protocol, connection_manager = setup_protocol
    transport = MagicMock()
    transport.get_extra_info.return_value = 'peername'

    protocol.connection_made(transport)

    assert protocol._transport == transport
    mock_make_parser.return_value.setContentHandler.assert_called()
    connection_manager.connection.assert_called_with('peername')
    assert protocol._timeout_monitor is not None

@patch('xml.sax.make_parser', return_value=MagicMock())
def test_connection_made_without_transport(mock_make_parser, setup_protocol):
    protocol, connection_manager = setup_protocol
    transport = None

    with patch('pyjabber.network.server.incoming.XMLServerIncomingProtocol.logger') as mock_logger:
        protocol.connection_made(transport)
        mock_logger.error.assert_called_with("Invalid transport")


def test_eof_received(setup_protocol):
    protocol, connection_manager = setup_protocol
    transport = MagicMock()
    transport.get_extra_info.return_value = 'peername'
    protocol._transport = transport
    protocol._xml_parser = MagicMock()

    protocol.eof_received()

    connection_manager.disconnection_server.assert_called_with('peername')
    assert protocol._transport is None
    assert protocol._xml_parser is None


@patch('xml.sax.make_parser', return_value=MagicMock())
def test_connection_made_with_timeout_monitor(mock_make_parser, setup_protocol):
    protocol, connection_manager = setup_protocol
    transport = MagicMock()
    transport.get_extra_info.return_value = 'peername'

    protocol.connection_made(transport)

    assert isinstance(protocol._timeout_monitor, StreamAlivenessMonitor)


if __name__ == "__main__":
    pytest.main()



import xml.etree.ElementTree as ET
from abc import ABC, abstractmethod


class Plugin(ABC):
    @abstractmethod
    def __init__(self, jid: str):
        self.jid = jid

    @abstractmethod
    def feed(self, element: ET.Element):
        pass



import pytest
from unittest.mock import patch, MagicMock
import xml.etree.ElementTree as ET
from pyjabber.features.presence.PresenceFeature import Presence
from pyjabber.features.feature_utils import RosterUtils as RU

@pytest.fixture
def setup_presence():
    with patch('pyjabber.features.feature_utils.RosterUtils.retrieve_roster') as mock_retrieve_roster:
        with patch('pyjabber.features.feature_utils.RosterUtils.update') as mock_update:
            with patch('pyjabber.network.ConnectionManager.ConnectionManager') as MockConnectionsManager:
                mock_connections = MockConnectionsManager.return_value
                jid = 'user2@localhost'
                presence = Presence(jid, mock_connections)
                yield presence, mock_connections, mock_retrieve_roster, mock_update

def elements_are_equal(e1, e2):
    if e1.tag != e2.tag or e1.attrib != e2.attrib or e1.text != e2.text:
        return False
    return all(elements_are_equal(c1, c2) for c1, c2 in zip(e1, e2))

def test_handle_subscribe(setup_presence):
    presence, mock_connections, mock_retrieve_roster, mock_update = setup_presence
    element = ET.Element('presence', attrib={'type': 'subscribe', 'to': 'user@localhost', 'id': '123'})

    # Simular la respuesta de retrieve_roster
    mock_retrieve_roster.side_effect = [
        [(1, 'user2@localhost', '<item jid="user@localhost" subscription="none"/>')],  # Simulación inicial
        [(1, 'user2@localhost', '<item jid="user@localhost" subscription="ask"/>')]  # Después de update
    ]

    # Simular la respuesta de update
    mock_update.return_value = '<item jid="user@localhost" subscription="ask"/>'

    mock_connections.get_buffer.return_value = [MagicMock()]

    presence._jid = 'user2@localhost'

    result = presence.handle_subscribe(element)

    assert result is None
    mock_retrieve_roster.assert_called_with('user2@localhost')

    # Comparar elementos XML
    expected_item = ET.Element('item', attrib={'jid': 'user@localhost', 'subscription': 'none', 'ask': 'subscribe'})
    actual_call = mock_update.call_args
    assert actual_call is not None
    actual_item = actual_call[1]['item']
    assert elements_are_equal(expected_item, actual_item)
    assert actual_call[1]['id'] == 1
def test_handle_subscribed(setup_presence):
    presence, mock_connections, mock_retrieve_roster, mock_update = setup_presence
    element = ET.Element('presence', attrib={'type': 'subscribed', 'to': 'user@localhost', 'id': '123'})

    presence._jid = 'user2@localhost'

    # Simular la respuesta de retrieve_roster
    mock_retrieve_roster.side_effect = [
        [(1, 'user2@localhost', '<item jid="user@localhost" subscription="from"/>')],
        [(1, 'user@localhost', '<item jid="user2@localhost" subscription="none"/>')]
    ]

    # Simular las respuestas de get_buffer_by_jid
    mock_connections.get_buffer_by_jid.side_effect = [
        [MagicMock()],
        [MagicMock()]
    ]

    result = presence.handle_subscribed(element)

    assert result is None



def test_handle_unsubscribed(setup_presence):
    presence, mock_connections, mock_retrieve_roster, mock_update = setup_presence
    element = ET.Element('presence', attrib={'type': 'unsubscribed', 'to': 'user@localhost', 'id': '123'})

    presence._jid = 'user2@localhost'

    # Simular la respuesta de retrieve_roster
    mock_retrieve_roster.side_effect = [
        [(1, 'user2@localhost', '<item jid="user@localhost" subscription="both"/>')]
    ]

    # Simular la respuesta de get_buffer_by_jid
    mock_connections.get_buffer.return_value = [MagicMock()]

    result = presence.handle_unsubscribed(element)

    assert result is None

def test_handle_unavailable(setup_presence):
    presence, mock_connections, mock_retrieve_roster, mock_update = setup_presence
    element = ET.Element('presence', attrib={'type': 'unavailable', 'from': 'user2@localhost', 'id': '123'})

    presence._jid = 'user2@localhost'  # Inicializar _jid

    mock_retrieve_roster.retriveRoster.return_value = [
        (1, 'user@localhost', '<item jid="user2@localhost" subscription="both"/>')
    ]

    mock_connections.get_buffer_by_jid.return_value = [MagicMock()]

    result = presence.handle_unavailable(element)

    assert result is None

def test_feed_handle_initial_presence(setup_presence):
    presence, mock_connections, mock_retrieve_roster, mock_update = setup_presence
    element = ET.Element('presence', attrib={'id': '123'})
    jid = 'user@localhost'

    mock_retrieve_roster.return_value = [
        (1, 'user2@localhost', '<item jid="user2@localhost" subscription="both"/>')
    ]

    buffer_mock = MagicMock()
    mock_connections.get_buffer.return_value = [buffer_mock]

    result = presence.feed(element)

    assert result is None
    mock_retrieve_roster.assert_called_once_with('user2@localhost')
    mock_connections.get_buffer.assert_called_once_with('user2@localhost')
    buffer_mock[-1].write.assert_called_once()



def test_handle_initial_presence(setup_presence):
    presence, mock_connections, mock_retrieve_roster, mock_update = setup_presence
    element = ET.Element('presence', attrib={'id': '123'})
    presence._jid = 'user@localhost'

    mock_retrieve_roster.return_value = [
        (1, 'user2@localhost', '<item jid="user2@localhost" subscription="both"/>')
    ]

    buffer_mock = MagicMock()
    mock_connections.get_buffer.return_value = [buffer_mock]

    presence.handle_initial_presence(element)

    mock_retrieve_roster.assert_called_once_with('user@localhost')
    mock_connections.get_buffer.assert_called_once_with('user2@localhost')
    buffer_mock[-1].write.assert_called_once()


if __name__ == "__main__":
    pytest.main()



import pytest
from xml.etree import ElementTree as ET
from unittest.mock import MagicMock

from pyjabber.features.StreamFeature import  StreamFeature,NonUniqueFeature


def test_initialization():
    stream_feature = StreamFeature()
    assert stream_feature.tag == "stream:features"
    assert stream_feature.attrib == {"xmlns": "http://etherx.jabber.org/streams"}
    assert stream_feature._features == {}

def test_register_feature():
    stream_feature = StreamFeature()
    feature = ET.Element("feature1")

    stream_feature.register(feature)

    assert "feature1" in stream_feature._features
    assert stream_feature._features["feature1"] == feature

def test_register_non_unique_feature():
    stream_feature = StreamFeature()
    feature = ET.Element("feature1")

    stream_feature.register(feature)

    with pytest.raises(NonUniqueFeature):
        stream_feature.register(feature)

def test_unregister_feature():
    stream_feature = StreamFeature()
    feature = ET.Element("feature1")

    stream_feature.register(feature)
    stream_feature.unregister(feature)

    assert "feature1" not in stream_feature._features

def test_unregister_nonexistent_feature():
    stream_feature = StreamFeature()
    feature = ET.Element("feature1")

    # Attempting to unregister a feature that doesn't exist should do nothing
    stream_feature.unregister(feature)

    assert "feature1" not in stream_feature._features

def test_reset_features():
    stream_feature = StreamFeature()
    feature1 = ET.Element("feature1")
    feature2 = ET.Element("feature2")

    stream_feature.register(feature1)
    stream_feature.register(feature2)

    stream_feature.reset()

    assert stream_feature._features == {}

def test_tostring():
    stream_feature = StreamFeature()
    feature = ET.Element("feature1")
    stream_feature.register(feature)

    xml_string = stream_feature.tostring()

    assert xml_string == '<stream:features xmlns="http://etherx.jabber.org/streams"><feature1 /></stream:features>'

def test_tobytes():
    stream_feature = StreamFeature()
    feature = ET.Element("feature1")
    stream_feature.register(feature)

    xml_bytes = stream_feature.to_bytes()

    expected_bytes = b'<stream:features xmlns="http://etherx.jabber.org/streams"><feature1 /></stream:features>'
    assert xml_bytes == expected_bytes




import pytest
from unittest.mock import MagicMock, patch
import xml.etree.ElementTree as ET

from pyjabber.features.presence.utils import create_roster_entry

@patch('pyjabber.features.presence.utils.uuid4')
def test_create_roster_entry(mock_uuid):
    # Configuramos el mock para uuid4
    mock_uuid.return_value = "1234-5678-uuid"

    mock_roster_manager = MagicMock()

    jid = "user1@localhost"
    to = "user2@localhost"

    create_roster_entry(jid, to, mock_roster_manager)

    expected_iq = ET.Element(
        "iq", attrib={"from": jid, "id": "1234-5678-uuid", "type": "set"}
    )
    query = ET.Element("{jabber:iq:roster}query")
    item = ET.Element("{jabber:iq:roster}item", attrib={"jid": to, "subscription": "none"})
    query.append(item)
    expected_iq.append(query)

    expected_iq_str = ET.tostring(expected_iq).decode()
    actual_iq_str = ET.tostring(mock_roster_manager.feed.call_args[0][0]).decode()

    assert expected_iq_str == actual_iq_str



import socket
from enum import Enum
from typing import Union
from uuid import uuid4
from xml.etree import ElementTree as ET

from pyjabber.features import InBandRegistration as IBR
from pyjabber.features.StartTLSFeature import StartTLSFeature
from pyjabber.features.StreamFeature import StreamFeature
from pyjabber.features.SASLFeature import SASLFeature, SASL
from pyjabber.features.ResourceBinding import ResourceBinding
from pyjabber.network.ConnectionManager import ConnectionManager
from pyjabber.utils import ClarkNotation as CN


class Stage(Enum):
    """
    Stream connection states.
    """
    CONNECTED = 0
    OPENED = 1
    SSL = 2
    SASL = 3
    AUTH = 4
    BIND = 5
    READY = 6


class Signal(Enum):
    RESET = 0
    DONE = 1

    def __eq__(self, other):
        if other is None:
            return False
        return self.value == other.value


class StreamHandler:
    def __init__(self, host, buffer, starttls, connection_manager) -> None:
        self._host = host
        self._buffer = buffer
        self._starttls = starttls

        self._streamFeature = StreamFeature()
        self._connection_manager: ConnectionManager = connection_manager
        self._stage = Stage.CONNECTED

        self._elem = None
        self._jid = None

    @property
    def buffer(self):
        return self._buffer

    @buffer.setter
    def buffer(self, value):
        self._buffer = value

    def handle_open_stream(self, elem: ET.Element = None) -> Union[Signal, None]:
        # TCP Connection opened
        if self._stage == Stage.CONNECTED:
            self._streamFeature.reset()
            self._streamFeature.register(StartTLSFeature())
            self._buffer.write(self._streamFeature.to_bytes())

            self._stage = Stage.OPENED
            return

        # TLS feature offered
        elif self._stage == Stage.OPENED:
            if "starttls" in elem.tag:
                self._buffer.write(StartTLSFeature().proceed_response())
                self._starttls()
                self._stage = Stage.SSL
                return Signal.RESET

            else:
                raise Exception()

        # TLS Handshake made. Starting SASL
        elif self._stage == Stage.SSL:
            self._streamFeature.reset()

            self._streamFeature.register(IBR.InBandRegistration())
            self._streamFeature.register(SASLFeature())
            self._buffer.write(self._streamFeature.to_bytes())

            self._stage = Stage.SASL

        # SASL
        elif self._stage == Stage.SASL:
            res = SASL(self._connection_manager).feed(elem, {"peername": self._buffer.get_extra_info('peername')})
            if type(res) is tuple:
                if res[0].value == Signal.RESET.value:
                    self._buffer.write(res[1])
                    self._stage = Stage.AUTH
                    return Signal.RESET
                else:
                    self._buffer.write(res[1])
                    self._stage = Stage.AUTH
                    return

            self._buffer.write(res)

        # User register/authenticated. Starting resource binding
        elif self._stage == Stage.AUTH:
            self._streamFeature.reset()
            self._streamFeature.register(ResourceBinding())
            self._buffer.write(self._streamFeature.to_bytes())

            self._stage = Stage.BIND

        elif self._stage == Stage.BIND:
            if "iq" in elem.tag:
                if elem.attrib["type"] == "set":
                    bindElem = elem.find(CN.clarkFromTuple(("urn:ietf:params:xml:ns:xmpp-bind", "bind")))
                    resouce = bindElem.find(CN.clarkFromTuple(("urn:ietf:params:xml:ns:xmpp-bind", "resource")))

                    if resouce is not None:
                        resource_id = resouce.text
                    else:
                        resource_id = uuid4()

                    iqRes = ET.Element(
                        "iq",
                        attrib={
                            "id": elem.attrib["id"],
                            "type": "result"
                        }
                    )

                    bindRes = ET.SubElement(
                        iqRes,
                        "bind",
                        attrib={
                            "xmlns": "urn:ietf:params:xml:ns:xmpp-bind"
                        }
                    )

                    jidRes = ET.SubElement(bindRes, "jid")

                    currentJid = self._connection_manager.get_jid(self._buffer.get_extra_info('peername'))
                    jidRes.text = f"{currentJid}@{self._host}/{resource_id}"

                    self._buffer.write(ET.tostring(iqRes))

                    # Stream is negotiated.
                    # Update the connection register
                    # with the jid and transport
                    self._connection_manager.set_jid(
                        self._buffer.get_extra_info('peername'),
                        jidRes.text, self._buffer
                    )

            return Signal.DONE



import re
from typing import List
from asyncio import Transport
from typing import Dict, Union, Tuple
from loguru import logger

from pyjabber.utils import Singleton


class ConnectionManager(metaclass=Singleton):
    JID = "jid"
    TRANSPORT = "transport"

    def __init__(self, task_s2s) -> None:
        self._task_s2s = task_s2s

        self._peerList = {}
        self._remoteList = {}

    ###########################################################################
    ############################### LOCAL BOUND ###############################
    ###########################################################################

    def get_users_connected(self) -> Dict[str, Tuple[str, int]]:
        """
            Return all the users connected

            :return: A dictionary of users connected in the format { (IP, PORT): {jid: <JID>, transport:
            <TRANSPORT> }}
        """
        return self._peerList

    def get_buffer(self, jid: str) -> List[Tuple[str, Transport]]:
        """
            Get all the available buffers associated with a jid.

            If the jid is in the format username@domain/resource, the function
            will only return one buffer or none.

            If the jid is in the format username@domain, the function
            will return a list of the buffers for each resource available.

            :param jid: The jid to get the buffers for. It can be a full jid or a bare jid
            :return: (<JID>, <TRANSPORT>) tuple
        """

        return [(self._peerList[key][self.JID], self._peerList[key][self.TRANSPORT])
                for key, values in self._peerList.items()
                if values[self.JID] is not None and re.match(f"{jid}/*", values[self.JID])
                ]

    def get_jid(self, peer) -> Union[str, None]:
        """
            Return the jid associated with the peername

            :param peer: The peer value in the tuple format ({IP}, {PORT})
        """
        try:
            return self._peerList[peer][self.JID]
        except KeyError:
            return None

    def set_jid(self, peer: Tuple[str, int], jid: str, transport: Transport = None) -> Union[None, bool]:
        """
            Set/update the jid of a registered connection.

            An optional transport argument can be provided, in order to
            set/update the stored buffer

            :param peer: The peer value in the tuple format ({IP}, {PORT})
            :param jid: The jid to set/update
            :param transport: Transport to use
        """
        try:
            self._peerList[peer][self.JID] = jid
            if transport:
                self._peerList[peer][self.TRANSPORT] = transport
        except KeyError:
            return False

    def connection(self, peer) -> None:
        """
            Store a new connection, without jid or transport.
            Those will be added in the future with the set_jid method.

            :param peer: The peer value in the tuple format ({IP}, {PORT})
        """
        if peer not in self._peerList:
            self._peerList[peer] = {
                self.JID: None,
                self.TRANSPORT: None
            }

    def disconnection(self, peer) -> None:
        """
            Remove a stored connection.

            :param peer: The peer value in the tuple format ({IP}, {PORT})
        """
        try:
            self._peerList.pop(peer)
        except KeyError:
            logger.error(f"{peer} not present in the online list")

    ###########################################################################
    ############################# REMOTE SERVER ###############################
    ###########################################################################

    def connection_server(self, peer, host, transport) -> None:
        """
            Store a new connection, without jid or transport.
        """
        if peer not in self._remoteList:
            self._remoteList[peer] = {
                self.JID: host,
                self.TRANSPORT: None
            }

    def disconnection_server(self, peer) -> None:
        """
        Remove a present server connection in the list
        i.e. EOF recived, or TCP connection lost
        """
        try:
            self._remoteList.pop(peer)
        except KeyError:
            logger.error(f"{peer} not present in the online list")

    def get_server_buffer(self, host: str) -> Union[Tuple[str, Transport], None]:
        """
            Return the buffer associated with the given host
            :return: (<HOST>, <TRANSPORT>) tuple
        """
        if self.check_server_stream_available(host):
            key = next((k for k, v in self._remoteList.items() if v.get(self.JID) == host), None)
            return self._remoteList[key][self.JID], self._remoteList[key][self.TRANSPORT]

        if self.check_server_present_in_list(host):
            return

        self._task_s2s(host)

    def get_server_host(self, peer: Tuple[str, int]):
        """
            Return the host associated with the given peer.
            :return: Hostname
        """
        try:
            return self._remoteList[peer]["host"]
        except KeyError:
            return None

    def set_server_transport(self, peer: Tuple[str, int], transport: Transport) -> Union[None, bool]:
        """
            Set/update the transport of a registered server connection.

            :param peer: The peer value in the tuple format ({IP}, {PORT})
            :param transport: Transport to use
        """
        try:
            self._remoteList[peer][self.TRANSPORT] = transport
        except KeyError:
            return False

    def set_server_host(self, peer: Tuple[str, int], host: str):
        """
            Set/update the host of a registered server connection.

            :param peer: The peer value in the tuple format ({IP}, {PORT})
            :param host: New host name
        """
        try:
            self._remoteList[peer]["host"] = host
        except KeyError:
            return False

    def check_server_stream_available(self, host) -> bool:
        for k, v in self._remoteList.items():
            if v[self.JID] == host:
                return v[self.TRANSPORT] is not None

    def check_server_present_in_list(self, host) -> bool:
        try:
            if [v for v in self._remoteList.values() if v[self.JID] == host]:
                return True
        except KeyError:
            return False



import pytest
import sqlite3
import xml.etree.ElementTree as ET
from pyjabber.plugins.roster.Roster import Roster
from pyjabber.stanzas.error import StanzaError as SE
@pytest.fixture
def setup_database():
    con = sqlite3.connect(':memory:')
    cur = con.cursor()
    cur.execute('''
        CREATE TABLE roster (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            jid VARCHAR(255) NOT NULL,
            rosterItem VARCHAR(255) NOT NULL
        )
    ''')
    cur.execute('''
        INSERT INTO roster (jid, rosterItem) VALUES
        ('jid1', '<item jid="jid1" name="name1" subscription="both"/>'),
        ('jid2', '<item jid="jid2" name="name2" subscription="none"/>')
    ''')
    con.commit()
    yield con
    con.close()

@pytest.fixture
def db_connection_factory(setup_database):
    def factory():
        return setup_database
    return factory



def test_feed_invalid_xml(db_connection_factory):
    roster = Roster('jid1', db_connection_factory)

    # Crear un elemento XML inválido (más de un hijo)
    element = ET.Element('element')
    child1 = ET.SubElement(element, 'child')
    child2 = ET.SubElement(element, 'child')

    # Verificar que se devuelve un error de XML inválido
    result = roster.feed(element)
    assert result == SE.invalid_xml()


def test_feed_handle_get(db_connection_factory):
    roster = Roster( 'jid1', db_connection_factory)

    # Mock del manejador 'get'
    def mock_handle_get(element):
        return '<result />'

    roster._handlers['get'] = mock_handle_get

    # Crear un elemento XML válido
    element = ET.Element('iq', attrib={'type': 'get'})
    child = ET.SubElement(element, 'query')

    # Verificar que el manejador 'get' se llama correctamente
    result = roster.feed(element)
    assert result == '<result />'


def test_handleGet_existing_jid(db_connection_factory):
    roster = Roster('jid1', db_connection_factory)

    element = ET.Element('iq', attrib={'id': '1234', 'type': 'get'})

    result = roster.handle_get(element)

    assert b'<iq id="1234" type="result">' in result  # Verificamos que el resultado contiene la respuesta IQ correcta

def test_handleGet_non_existing_jid(db_connection_factory):
    roster = Roster('non_existing_jid', db_connection_factory)
    element = ET.Element('iq', attrib={'id': '1234', 'type': 'get'})

    result = roster.handle_get(element)

    assert b'<iq id="1234" type="result">' in result  # Verificamos que se maneja adecuadamente aunque el jid no exista



###########
#Handle SET
###########

def test_handleSet_add_new_item(db_connection_factory):
    roster = Roster('jid3', db_connection_factory)

    element = ET.Element('iq', attrib={'id': '1234', 'type': 'set'})
    # Definimos correctamente el elemento 'query' con el espacio de nombres correcto
    query = ET.SubElement(element, '{jabber:iq:roster}query')
    item = ET.SubElement(query, '{jabber:iq:roster}item',
                         attrib={'jid': 'jid3', 'name': 'name3', 'subscription': 'both'})


    result = roster.handle_set(element)

    assert b'<iq id="1234" type="result" />' in result



def test_handleSet_update_existing_item(db_connection_factory):
    roster = Roster('jid1',db_connection_factory)

    element = ET.Element('iq', attrib={'id': '1234', 'type': 'set'})
    # Definimos correctamente el elemento 'query' con el espacio de nombres correcto
    query = ET.SubElement(element, '{jabber:iq:roster}query')
    item = ET.SubElement(query, '{jabber:iq:roster}item',
                         attrib={'jid': 'jid3', 'name': 'name3', 'subscription': 'both'})


    result = roster.handle_set(element)

    assert b'<iq id="1234" type="result" />' in result


def test_handleSet_remove_item(db_connection_factory):
    roster = Roster('jid1',db_connection_factory)

    element = ET.Element('iq', attrib={'id': '1234', 'type': 'set'})
    # Definimos correctamente el elemento 'query' con el espacio de nombres correcto
    query = ET.SubElement(element, '{jabber:iq:roster}query')
    item = ET.SubElement(query, '{jabber:iq:roster}item',
                         attrib={'jid': 'jid3', 'name': 'name3', 'subscription': 'both'})


    result = roster.handle_set(element)

    assert b'<iq id="1234" type="result" />' in result
def test_handleSet_invalid_xml(db_connection_factory):
    roster = Roster('jid1', db_connection_factory)

    # Create an element with multiple items
    element = ET.Element('iq', attrib={'id': '1234', 'type': 'set'})
    query = ET.SubElement(element, '{jabber:iq:roster}query')
    ET.SubElement(query, '{jabber:iq:roster}item', attrib={'jid': 'jid1', 'subscription': 'both'})
    ET.SubElement(query, '{jabber:iq:roster}item', attrib={'jid': 'jid2', 'subscription': 'none'})

    with pytest.raises(Exception):
        roster.handle_set(element)

def test_handleSet_missing_query(db_connection_factory):
    roster = Roster('jid1', db_connection_factory)

    element = ET.Element('iq', attrib={'id': '1234', 'type': 'set'})

    with pytest.raises(Exception):
        roster.handleSet(element)

def test_handleSet_no_items(db_connection_factory):
    roster = Roster('jid1', db_connection_factory)

    element = ET.Element('iq', attrib={'id': '1234', 'type': 'set'})
    query = ET.SubElement(element, '{jabber:iq:roster}query')

    with pytest.raises(Exception):
        roster.handleSet(element)



from loguru import logger
from xml.etree import ElementTree as ET

from pyjabber.network.XMLParser import XMLParser
from pyjabber.stream import Stream
from pyjabber.stream.StanzaHandler import StanzaHandler
from pyjabber.stream.StreamHandler import Signal
from pyjabber.stream.server.outcoming.StreamServerOutcomingHandler import StreamServerOutcomingHandler
from pyjabber.utils import ClarkNotation as CN


class XMLServerOutcomingParser(XMLParser):
    """
    Manages the stream data and process the XML objects.
    Inheriting from sax.ContentHandler
    """

    def __init__(
            self,
            buffer,
            starttls,
            connection_manager,
            queue_message,
            host,
            public_host):

        super().__init__(host, buffer, starttls, connection_manager, queue_message)

        self._host = host
        self._public_host = public_host
        self._streamHandler = StreamServerOutcomingHandler(public_host, buffer, starttls, connection_manager)
        self.initial_stream()

    def startElementNS(self, name, qname, attrs):
        logger.debug(f"Start element NS: {name}")

        CN.clarkFromTuple(name)
        if CN.clarkFromTuple(name) == '{http://etherx.jabber.org/streams}stream' and self._stack:
            # ERROR Stream already present in stack
            raise Exception()

        elem = ET.Element(
            CN.clarkFromTuple(name),
            attrib={CN.clarkFromTuple(key): item for key, item in dict(attrs).items()}
        )
        self._stack.append(elem)

    def endElementNS(self, name, qname):
        logger.debug(f"End element NS: {qname} : {name}")

        if name == "</stream:stream>":
            self._buffer.write(b'</stream>')
            self._stack.clear()
            return

        if not self._stack:
            raise Exception()

        elem = self._stack.pop()

        if elem.tag != CN.clarkFromTuple(name):
            # INVALID STANZA/MESSAGE
            raise Exception()

        if self._stack[-1].tag != '{http://etherx.jabber.org/streams}stream':
            self._stack[-1].append(elem)

        else:
            if self._state == self.StreamState.READY:  # Ready to process stanzas
                self._stanzaHandler.feed(elem)
            else:
                signal = self._streamHandler.handle_open_stream(elem)
                if signal == Signal.RESET:
                    self._stack.clear()
                    self.initial_stream()
                elif signal == Signal.DONE:
                    self._stanzaHandler = StanzaHandler(self._buffer, self._connection_manager, None)
                    self._state = self.StreamState.READY

    def initial_stream(self):
        initial_stream = Stream.Stream(
            from_=self._public_host,
            to=self._host,
            xmlns=Stream.Namespaces.SERVER.value
        )

        initial_stream = initial_stream.open_tag()
        self._buffer.write(initial_stream)



from pyjabber.features.StartTLSFeature import StartTLSFeature
import pytest
from xml.etree import ElementTree as ET
from enum import Enum

class Signal(Enum):
    RESET = 0
    DONE = 1

class FeatureInterface:
    def feed(self, element: ET.Element):
        pass

class TLS(FeatureInterface):
    def __init__(self):
        pass

    def feed(self, element: ET.Element):
        pass


def test_initialization_required():
    starttls_feature = StartTLSFeature()
    assert starttls_feature.tag == "starttls"
    assert starttls_feature.attrib == {"xmlns": "urn:ietf:params:xml:ns:xmpp-tls"}
    assert len(starttls_feature) == 1
    assert starttls_feature[0].tag == "required"

def test_initialization_not_required():
    starttls_feature = StartTLSFeature(required=False)
    assert starttls_feature.tag == "starttls"
    assert starttls_feature.attrib == {"xmlns": "urn:ietf:params:xml:ns:xmpp-tls"}
    assert len(starttls_feature) == 0

def test_proceed_response():
    starttls_feature = StartTLSFeature()
    proceed_response = starttls_feature.proceed_response()
    expected_response = b'<proceed xmlns="urn:ietf:params:xml:ns:xmpp-tls" />'
    assert proceed_response == expected_response

if __name__ == "__main__":
    pytest.main()




from enum import Enum
from typing import Dict
from xml.etree import ElementTree as ET

from pyjabber.features.FeatureInterface import FeatureInterface


class Signal(Enum):
    RESET = 0
    DONE = 1


class TLS(FeatureInterface):
    def __init__(self):
        pass

    def feed(self, element: ET.Element):
        pass


class StartTLSFeature(ET.Element):
    def __init__(
        self,
        tag: str = "starttls",
        attrib: Dict[str, str] = {
            "xmlns": "urn:ietf:params:xml:ns:xmpp-tls"
        },
        required: bool = True,
            **extra: str) -> None:
        super().__init__(tag, attrib, **extra)

        if required:
            self.append(ET.Element("required"))

    @staticmethod
    def proceed_response() -> bytes:
        elem = ET.Element(
            "proceed", attrib={
                "xmlns": "urn:ietf:params:xml:ns:xmpp-tls"})
        return ET.tostring(elem)



from unittest.mock import MagicMock, patch
from xml.etree.ElementTree import Element
import xml.etree.ElementTree as ET
import pickle
from pyjabber.utils import ClarkNotation as CN
import os
from pyjabber.stanzas.error import StanzaError as SE
from pyjabber.stream.StanzaHandler import StanzaHandler

def setUp():
    mock_buffer = MagicMock()
    mock_buffer.get_extra_info.return_value = '127.0.0.1'
    mock_queue_message = MagicMock()
    mock_host = "domain.es"

    with patch('pyjabber.network.ConnectionManager.ConnectionManager') as MockConnectionsManager, \
         patch('pyjabber.features.presence.PresenceFeature.Presence') as MockPresence:

        mock_connections = MockConnectionsManager.return_value
        mock_presence = MockPresence.return_value
        mock_connections.get_jid.return_value = 'user@domain.com'

        schemas_path = os.path.join(os.path.dirname(__file__), '..', 'pyjabber', 'stream', 'schemas', 'schemas.pkl')
        with open(schemas_path, 'rb') as f:
            schemas = pickle.load(f)

        handler = StanzaHandler(mock_host, mock_buffer, mock_connections, mock_queue_message)
        handler._host = mock_host
        handler._schemas = schemas
        handler._connections = mock_connections
        handler._presenceManager = mock_presence
        handler._functions = {
            "{jabber:client}iq": MagicMock(),
            "{jabber:client}message": MagicMock(),
            "{jabber:client}presence": MagicMock()
        }

    return handler, mock_buffer, mock_connections, mock_presence, mock_queue_message

def test_feed_valid_element():
    handler, mock_buffer, mock_connections, mock_presence, mock_queue_message = setUp()
    element = Element('iq', attrib={"to": "localhost", "type": "get"})
    element.tag = "{jabber:client}iq"
    child = Element('query')
    element.append(child)

    # Simula que el esquema es válido
    schema = handler._schemas[CN.deglose(element.tag)[0]]
    with patch.object(schema, 'is_valid', return_value=True):
        handler.feed(element)

    handler._functions[element.tag].assert_called_once_with(element)

@patch('pyjabber.stanzas.error.StanzaError.bad_request')
def test_feed_invalid_element_schema(mock_bad_request):
    handler, mock_buffer, mock_connections, mock_presence, mock_queue_message = setUp()
    element = Element('iq', attrib={"to": "localhost", "type": "get"})
    element.tag = "{jabber:client}iq"
    child = Element('{jabber:client}query')
    element.append(child)
    schema = handler._schemas[CN.deglose(element.tag)[0]]
    with patch.object(schema, 'is_valid', return_value=False):
        handler.feed(element)
    mock_bad_request.assert_called_once()
    mock_buffer.write.assert_called_once_with(mock_bad_request())


@patch('pyjabber.stanzas.error.StanzaError.feature_not_implemented')
def test_feed_function_key_error(mock_feature_not_implemented):
    handler, mock_buffer, mock_connections, mock_presence, mock_queue_message = setUp()
    element = Element('unknown', attrib={"to": "localhost"})
    element.tag = "{jabber:client}unknown"
    child = Element('{jabber:client}query')
    element.append(child)

    # Simula que el esquema es válido
    schema = handler._schemas[CN.deglose(element.tag)[0]]
    with patch.object(schema, 'is_valid', return_value=True):
        try:
            handler.feed(element)
        except Exception:
            # Verifica que la excepción se lanzó
            pass
        else:
            raise AssertionError("Exception not raised")

    # No debería llegar aquí porque se debe lanzar una excepción
    mock_feature_not_implemented.assert_not_called()
    mock_buffer.write.assert_not_called()

@patch('pyjabber.stanzas.error.StanzaError.feature_not_implemented')
def test_feed_key_error_in_schemas(mock_feature_not_implemented):
    handler, mock_buffer, mock_connections, mock_presence, mock_queue_message = setUp()
    element = Element('presence', attrib={"to": "localhost"})
    element.tag = "{jabber:client}unknown"

    with patch.dict(handler._schemas, {}, clear=True):
        handler.feed(element)

    mock_feature_not_implemented.assert_called_once()
    mock_buffer.write.assert_called_once_with(mock_feature_not_implemented())

def test_handleIQ():
    handler, mock_buffer, mock_connections, mock_presence, mock_queue_message = setUp()
    element = Element('iq', attrib={"to": "localhost", "type": "get"})
    child = Element('{jabber:client}query')  # Aseguramos que tenga el namespace adecuado
    element.append(child)

    expected_response = SE.service_unavaliable()

    with patch('pyjabber.plugins.PluginManager.PluginManager') as MockPluginManager:
        mock_plugin_manager = MockPluginManager.return_value
        mock_plugin_manager.feed.return_value = expected_response  # Simulamos la respuesta esperada
        handler.handle_iq(element)
        mock_buffer.write.assert_called_once_with(expected_response)


def test_handleMsg():
    handler, mock_buffer, mock_connections, mock_presence, mock_queue_message = setUp()
    element = Element('message', attrib={"to": "user@domain.com"})
    element.tag = "{jabber:client}message"

    mock_server_buffer = MagicMock()
    mock_connections.get_server_buffer.return_value = (None, mock_server_buffer)

    handler.handle_msg(element)

    mock_server_buffer.write.assert_called_once_with(ET.tostring(element))


def test_handleMsg_remote_user():
    handler, mock_buffer, mock_connections, mock_presence, mock_queue_message = setUp()
    element = Element('message', attrib={"to": "user@remote.com"})
    element.tag = "{jabber:client}message"
    mock_connections.get_server_buffer.return_value = None
    handler.handle_msg(element)
    mock_queue_message.enqueue.assert_called_once_with('remote.com', ET.tostring(element))

def test_handlePre():
    handler, mock_buffer, mock_connections, mock_presence, mock_queue_message = setUp()
    element = Element('presence', attrib={"to": "localhost"})
    mock_presence.feed.return_value = 'response'
    handler.handle_pre(element)
    mock_buffer.write.assert_called_once_with('response')



import hashlib
import json
from contextlib import closing

from aiohttp import web
from loguru import logger

from pyjabber.db.database import connection


async def handleUser(request):
    with closing(connection()) as con:
        res = con.execute("SELECT id, jid FROM credentials")
        res = res.fetchall()

    users = [{"id": i, "jid": v} for i, v in res]
    return web.Response(text=json.dumps(users))

async def handleRoster(request):
    try:
        user_id = int(request.match_info['id'])

        with closing(connection()) as con:
            res = con.execute("SELECT jid FROM credentials WHERE id = ?", (user_id, ))
            user_jid = res.fetchone()[0]

            if not user_jid:
                return web.json_response({"status": "error", "message": "User not found"}, status=404)

            res = con.execute("SELECT rosterItem FROM roster WHERE jid LIKE ?", (f"{user_jid}%", ))
            roster = res.fetchall()

            response = [{"item": r[0]} for r in roster]

        return web.Response(text=json.dumps(response))

    except ValueError:
        return web.json_response({"status": "error", "message": "Invalid user ID"}, status=400)
    except Exception as e:
        logger.error(e)
        error_response = {
            "status": "error",
            "message": str(e)
        }
        return web.json_response(error_response, status=500)

async def handleDelete(request):
    try:
        user_id = int(request.match_info['id'])

        with closing(connection()) as con:
            res = con.execute("SELECT id FROM credentials")
            res = res.fetchall()

            if user_id not in [r[0] for r in res]:
                return web.json_response({"status": "error", "message": "User not found"}, status=404)

            res = con.execute("SELECT jid FROM credentials WHERE id = ?", (user_id, ))
            user_jid = res.fetchone()[0]

            con.execute("DELETE FROM credentials WHERE id = ?", (user_id, ))
            con.execute("DELETE FROM roster WHERE jid LIKE ?", (f"{user_jid}%", ))
            con.commit()

        logger.info(f"User with ID {user_id} deleted")
        return web.json_response({"status": "success", "message": "User deleted"}, status=200)

    except ValueError:
        return web.json_response({"status": "error", "message": "Invalid user ID"}, status=400)
    except Exception as e:
        error_response = {
            "status": "error",
            "message": str(e)
        }
        return web.json_response(error_response, status=500)

async def handleRegister(request):
    try:
        data = await request.json()

        with closing(connection()) as con:
            res = con.execute("SELECT jid FROM credentials")
            res = res.fetchall()

        if any(r[0] == data["jid"] for r in res):
            raise Exception("JID already in use")

        hash_pwd = hashlib.sha256(data["pwd"].encode()).hexdigest()

        with closing(connection()) as con:
            con.execute("INSERT INTO credentials (jid, hash_pwd) VALUES (?, ?)",
                        (data["jid"], hash_pwd))
            con.commit()

        response_data = {
            "status": "success",
            "message": "OK",
            "received_data": data
        }

        logger.info(f"User with jid {data['jid']} created")
        return web.json_response(response_data, status=200)

    except Exception as e:
        error_response = {
            "status": "error",
            "message": str(e)
        }
        return web.json_response(error_response, status=500)



from xml.etree import ElementTree as ET

from pyjabber.plugins.PluginInterface import Plugin


class Ping(Plugin):
    def __init__(self, jid: str) -> None:
        self._jid = jid

    def feed(self, element: ET.Element):
        if "to" in element.attrib and element.attrib["to"] == "localhost":
            res = ET.tostring(
                ET.Element(
                    "iq",
                    attrib={
                        "from": "localhost",
                        "id": element.attrib["id"],
                        "to": element.attrib["to"],
                        "type": "result",
                    },
                )
            )

            return [res]



import os
import socket
from loguru import logger

from cryptography import x509
from cryptography.x509.oid import NameOID
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
import datetime


def check_hostname_cert_exists(host: str):
    previous_path = os.getcwd()
    file_path = os.path.dirname(os.path.abspath(__file__))

    os.chdir(os.path.join(file_path, "certs"))

    for file in [f"{host}_key.pem", f"{host}_csr.pem", f"{host}_cert.pem"]:
        if os.path.isfile(file) is False:
            os.chdir(os.path.join(previous_path))
            logger.debug("Missing hostname certificate")
            return False

    os.chdir(os.path.join(previous_path))
    return True


def generate_hostname_cert(host):
    logger.debug("Generating hostname certificate")
    previous_path = os.getcwd()
    file_path = os.path.dirname(os.path.abspath(__file__))

    os.chdir(os.path.join(file_path, "certs"))

    for file in [f"{host}_key.pem", f"{host}_csr.pem", f"{host}_cert.pem"]:
        if os.path.isfile(file):
            os.remove(file)

    domain_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
        backend=default_backend()
    )

    with open(f"{host}_key.pem", "wb") as f:
        f.write(domain_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()
        ))

    domain_name = host
    csr = x509.CertificateSigningRequestBuilder().subject_name(x509.Name([
        x509.NameAttribute(NameOID.ORGANIZATION_NAME, u"Spade"),
        x509.NameAttribute(NameOID.COMMON_NAME, domain_name),
    ])).add_extension(
        x509.SubjectAlternativeName([x509.DNSName(domain_name)]),
        critical=False,
    ).sign(domain_key, hashes.SHA256(), default_backend())

    with open(f"{host}_csr.pem", "wb") as f:
        f.write(csr.public_bytes(serialization.Encoding.PEM))

    with open("ca_key.pem", "rb") as f:
        ca_key = serialization.load_pem_private_key(
            f.read(),
            password=None,
            backend=default_backend()
        )

    with open("ca_cert.pem", "rb") as f:
        ca_cert = x509.load_pem_x509_certificate(f.read(), default_backend())

    domain_cert = x509.CertificateBuilder().subject_name(
        csr.subject
    ).issuer_name(
        ca_cert.subject
    ).public_key(
        csr.public_key()
    ).serial_number(
        x509.random_serial_number()
    ).not_valid_before(
        datetime.datetime.utcnow()
    ).not_valid_after(
        datetime.datetime.utcnow() + datetime.timedelta(days=365 * 10)
    ).add_extension(
        x509.SubjectAlternativeName([x509.DNSName(domain_name)]),
        critical=False,
    ).sign(ca_key, hashes.SHA256(), default_backend())

    with open(f"{host}_cert.pem", "wb") as f:
        f.write(domain_cert.public_bytes(serialization.Encoding.PEM))

    os.chdir(previous_path)

    logger.debug(f"Certificate generated for => {socket.gethostname()}")



import pytest
from unittest.mock import MagicMock, patch
from xml.etree.ElementTree import Element
from pyjabber.stream.server.incoming.StanzaServerIncomingHandler import StanzaServerIncomingHandler
from pyjabber.stream.server.incoming.StreamServerIncomingHandler import StreamServerIncomingHandler
from pyjabber.utils import ClarkNotation as CN
from pyjabber.network.server.incoming.XMLServerIncomingParser import XMLServerIncomingParser, StreamState, Signal

@pytest.fixture
def setup_parser():
    host = 'host'
    buffer = MagicMock()
    starttls = MagicMock()
    connection_manager = MagicMock()
    queue_message = MagicMock()
    parser = XMLServerIncomingParser(host, buffer, starttls, connection_manager, queue_message)
    return parser, buffer, connection_manager

def test_initialization(setup_parser):
    parser, buffer, connection_manager = setup_parser
    assert isinstance(parser._streamHandler, StreamServerIncomingHandler)

@patch.object(StreamServerIncomingHandler, 'handle_open_stream', MagicMock())
def test_start_element_ns_stream_start(setup_parser):
    parser, buffer, connection_manager = setup_parser
    name = ("http://etherx.jabber.org/streams", "stream")
    attrs = {("http://www.w3.org/XML/1998/namespace", "lang"): "en"}

    parser.startElementNS(name, None, attrs)

    buffer.write.assert_called()
    parser._streamHandler.handle_open_stream.assert_called()

def test_start_element_ns_nested_element(setup_parser):
    parser, buffer, connection_manager = setup_parser
    name = ("namespace", "element")
    attrs = {("namespace", "attr"): "value"}

    parser._stack.append(Element('dummy'))

    parser.startElementNS(name, None, attrs)
    assert len(parser._stack) == 2
    assert parser._stack[-1].tag == CN.clarkFromTuple(name)

def test_start_element_ns_exception(setup_parser):
    parser, buffer, connection_manager = setup_parser
    name = ("wrong_namespace", "element")

    with pytest.raises(Exception):
        parser.startElementNS(name, None, {})

def test_end_element_ns_stream_end(setup_parser):
    parser, buffer, connection_manager = setup_parser
    name = "</stream:stream>"

    parser.endElementNS(name, None)
    buffer.write.assert_called_with(b'</stream>')
    assert not parser._stack

def test_end_element_ns_no_stack_exception(setup_parser):
    parser, buffer, connection_manager = setup_parser
    name = ("namespace", "element")

    with pytest.raises(Exception):
        parser.endElementNS(name, None)

def test_end_element_ns_invalid_stanza_exception(setup_parser):
    parser, buffer, connection_manager = setup_parser
    parser._stack.append(Element('wrong_tag'))
    name = ("namespace", "element")

    with pytest.raises(Exception):
        parser.endElementNS(name, None)

def test_end_element_ns_append_to_stack(setup_parser):
    parser, buffer, connection_manager = setup_parser
    parent = Element('parent')
    child = Element(CN.clarkFromTuple(("namespace", "element")))

    parser._stack.append(parent)
    parser._stack.append(child)
    parser.endElementNS(("namespace", "element"), None)
    assert len(parent) == 1
    assert parent[0] == child


@patch('pyjabber.stream.server.incoming.StanzaServerIncomingHandler.Presence', MagicMock())
@patch.object(StreamServerIncomingHandler, 'handle_open_stream', return_value=Signal.DONE)
def test_end_element_ns_handle_open_stream(mock_handle_open_stream, setup_parser):
    parser, buffer, connection_manager = setup_parser
    parser._stack.append(Element('{http://etherx.jabber.org/streams}stream'))
    parser._state = StreamState.CONNECTED

    elem = Element(CN.clarkFromTuple(("namespace", "element")))
    parser._stack.append(elem)

    parser.endElementNS(("namespace", "element"), None)
    mock_handle_open_stream.assert_called()
    assert parser._state == StreamState.READY
    assert isinstance(parser._stanzaHandler, StanzaServerIncomingHandler)

def test_end_element_ns_signal_reset(setup_parser):
    parser, buffer, connection_manager = setup_parser
    parser._stack.append(Element('{http://etherx.jabber.org/streams}stream'))
    parser._state = StreamState.CONNECTED

    elem = Element(CN.clarkFromTuple(("namespace", "element")))
    parser._stack.append(elem)

    with patch.object(parser._streamHandler, 'handle_open_stream', return_value=Signal.RESET):
        parser.endElementNS(("namespace", "element"), None)
        assert not parser._stack

def test_end_element_ns_ready_state_feed(setup_parser):
    parser, buffer, connection_manager = setup_parser
    parser._stack.append(Element('{http://etherx.jabber.org/streams}stream'))
    parser._state = StreamState.READY
    parser._stanzaHandler = MagicMock()

    elem = Element(CN.clarkFromTuple(("namespace", "element")))
    parser._stack.append(elem)

    parser.endElementNS(("namespace", "element"), None)
    parser._stanzaHandler.feed.assert_called_with(elem)



import pytest
import xml.etree.ElementTree as ET

from pyjabber.stanzas.Presence import Presence


def test_presence_initialization():
    tag = "presence"
    attrib = {"type": "available"}
    extra = {"from": "user@example.com"}

    presence = Presence(tag=tag, attrib=attrib, **extra)

    assert presence.tag == tag
    assert presence.attrib["type"] == "available"
    assert presence.attrib["from"] == "user@example.com"

    assert isinstance(presence, ET.Element)

if __name__ == "__main__":
    pytest.main()



import os
import socket
import sys

import click
from loguru import logger

from pyjabber.server import Server


@click.command
@click.option('--host', type=str, default=socket.gethostname(),
              show_default=True, help='Host name')
@click.option('--client_port', type=int, default=5222,
              show_default=True, help='Server-to-client port')
@click.option('--server_port', type=int, default=5269,
              show_default=True, help='Server-to-server port')
@click.option('--family',
              type=click.Choice(['ipv4',
                                 'ipv6'],
                                case_sensitive=False),
              default='ipv4',
              show_default=True,
              help='(ipv4 / ipv6)')
@click.option('--tls1_3', is_flag=True, help='Enables TLSv1_3')
@click.option('--timeout', type=int, default=60,
              show_default=True, help='Timeout for connection')
@click.option('--log_level',
              type=click.Choice(['INFO',
                                 'DEBUG'],
                                case_sensitive=False),
              default='INFO',
              show_default=True,
              help='Log level alert')
@click.option('--log_path', type=str, help='Path to log dumpfile')
@click.option('--debug', '-D', is_flag=True,
              help='Enables debug mode in Asyncio')

def main(
        host,
        client_port,
        server_port,
        family,
        tls1_3,
        timeout,
        log_level,
        log_path,
        debug):

    log_file = os.devnull

    if log_path:
        log_file = open(os.path.join(log_path, "pyjabber.log"), 'w')

    logger.add(
        log_file,
        enqueue=True,
        format="<green>{time}</green> - <level>{level}: {message}</level>",
        level=log_level,
    )
    logger.configure(handlers=[{"sink": sys.stderr, "level": log_level}])

    server = Server(
        host=host,
        client_port=client_port,
        server_port=server_port,
        family=socket.AF_INET if family == "ipv4" else socket.AF_INET6,
        connection_timeout=timeout,
        enable_tls1_3=tls1_3,
    )

    server.start(debug)

    return 0


"""Allow cookiecutter to be executable through `python -m vangare`."""

if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



import xml.etree.ElementTree as ET
from enum import Enum


class IQ(ET.Element):
    class TYPE(Enum):
        GET = "get"
        RESULT = "result"
        SET = "set"
        ERROR = "error"

    def __init__(
            self,
            type: TYPE,
            id: str = None,
            from_: str = None,
            to: str = None,
            tag: str = "iq",
            **extra: str) -> None:

        attrib = {
            k: v for k, v in (
                ("id", id),
                ("from", from_),
                ("to", to),
                ("type", type)) if v is not None
        }

        super().__init__(tag, attrib, **extra)



import asyncio
import pytest
from unittest.mock import MagicMock, patch
from pyjabber.network.StreamAlivenessMonitor import StreamAlivenessMonitor


@pytest.fixture
def monitor():
    callback = MagicMock()
    mon = StreamAlivenessMonitor(timeout=1, callback=callback)
    return mon, callback


@pytest.mark.asyncio
async def test_timeout_callback_called(monitor):
    mon, callback = monitor
    mon.reset()  # Asegúrate de iniciar el monitor

    await asyncio.sleep(1.5)

    try:
        callback.assert_called_once()
    except AssertionError as e:
        print(f"AssertionError: {e}")
        print(f"Callback call count: {callback.call_count}")
        print(f"Was the timeout coroutine awaited?: {mon._timeout_task is not None and not mon._timeout_task.done()}")
        raise e




def test_monitor_initialization(monitor):
    mon, _ = monitor
    assert mon._timeout_task is None

@pytest.mark.asyncio
async def test_monitor_reset(monitor):
    mon, _ = monitor
    mon.reset()
    assert mon._timeout_task is not None
    assert not mon._timeout_task.done()



@pytest.mark.asyncio
async def test_reset(monitor):
    mon, callback = monitor
    mon.reset()
    assert mon._timeout_task is not None

    await asyncio.sleep(0.5)
    mon.reset()
    await asyncio.sleep(0.5)
    callback.assert_not_called()

    await asyncio.sleep(1.5)
    callback.assert_called_once()

@pytest.mark.asyncio
@patch('pyjabber.network.StreamAlivenessMonitor.asyncio.create_task')
async def test_reset_creates_task(mock_create_task, monitor):
    mon, callback = monitor
    mon.reset()
    await asyncio.sleep(0.1)
    mock_create_task.assert_called_once()

    called_coroutine = mock_create_task.call_args[0][0]
    assert asyncio.iscoroutine(called_coroutine)
    assert called_coroutine.__name__ == mon._timeout_task_coro().__name__


@pytest.fixture(autouse=True)
def cleanup(monitor):
    yield
    mon, callback = monitor
    if mon._timeout_task is not None:
        mon._timeout_task.cancel()




import unittest
from xml.etree import ElementTree as ET
from pyjabber.stanzas.Message import Message

def test_message_creation():
    mto = "user@domain.com"
    mfrom = "otheruser@domain.com"
    id = "12345"
    body = "Hello, this is a test message."
    mtype = "chat"
    tag = "message"

    msg = Message(mto=mto, mfrom=mfrom, id=id, body=body, mtype=mtype, tag=tag)

    # Verifica los atributos
    assert msg.attrib["to"] == mto
    assert msg.attrib["from"] == mfrom
    assert msg.attrib["id"] == id
    assert msg.attrib["type"] == mtype
    assert msg.tag == tag

    # Verifica el cuerpo del mensaje
    body_elem = msg.find("body")
    assert body_elem is not None
    assert body_elem.text == body

def test_message_creation_with_default_type():
    mto = "user@domain.com"
    mfrom = "otheruser@domain.com"
    id = "12345"
    body = "Hello, this is a test message."
    tag = "message"

    msg = Message(mto=mto, mfrom=mfrom, id=id, body=body, tag=tag)

    # Verifica los atributos
    assert msg.attrib["to"] == mto
    assert msg.attrib["from"] == mfrom
    assert msg.attrib["id"] == id
    assert msg.attrib["type"] == "chat"  # Valor por defecto
    assert msg.tag == tag

    # Verifica el cuerpo del mensaje
    body_elem = msg.find("body")
    assert body_elem is not None
    assert body_elem.text == body

def test_message_creation_with_extra_attributes():
    mto = "user@domain.com"
    mfrom = "otheruser@domain.com"
    id = "12345"
    body = "Hello, this is a test message."
    mtype = "chat"
    tag = "message"
    extra = {"lang": "en"}

    msg = Message(mto=mto, mfrom=mfrom, id=id, body=body, mtype=mtype, tag=tag, **extra)

    # Verifica los atributos
    assert msg.attrib["to"] == mto
    assert msg.attrib["from"] == mfrom
    assert msg.attrib["id"] == id
    assert msg.attrib["type"] == mtype
    assert msg.attrib["lang"] == "en"  # Extra attribute
    assert msg.tag == tag

    # Verifica el cuerpo del mensaje
    body_elem = msg.find("body")
    assert body_elem is not None
    assert body_elem.text == body





import base64
import hashlib
from contextlib import closing
from enum import Enum
from typing import Dict, List, Tuple, Union
from xml.etree import ElementTree as ET

from pyjabber.db.database import connection
from pyjabber.features.FeatureInterface import FeatureInterface
from pyjabber.stanzas.error import StanzaError as SE
from pyjabber.utils import ClarkNotation as CN


class mechanismEnum(Enum):
    PLAIN = "PLAIN"
    SCRAM_SHA_1 = "SCRAM-SHA-1"
    EXTERNAL = "EXTERNAL"


class Signal(Enum):
    RESET = 0
    DONE = 1





class SASL(FeatureInterface):
    def __init__(self, connection_manager, db_connection_factory=connection):
        self._handlers = {
            "iq": self.handleIQ,
            "auth": self.handleAuth
        }
        self._ns = "jabber:iq:register"
        self._connection_manager = connection_manager
        self._db_connection_factory = db_connection_factory
        self._peername = None

    def feed(self,
             element: ET,
             extra: Dict[str,
                         any] = None) -> Union[Tuple[Signal,
                                                     bytes],
                                               bytes]:
        if extra and "peername" in extra.keys():
            self._peername = extra["peername"]

        _, tag = CN.deglose(element.tag)
        return self._handlers[tag](element)

    def handleIQ(
            self, element: ET.Element) -> Union[Tuple[Signal, bytes], bytes]:
        query = element.find(CN.clarkFromTuple((self._ns, "query")))

        if query is None:
            raise Exception()

        elif element.attrib["type"] == "set":
            new_jid = query.find(
                CN.clarkFromTuple(
                    (self._ns, "username"))).text

            with closing(self._db_connection_factory()) as con:
                res = con.execute(
                    "SELECT * FROM credentials WHERE jid = ?", (new_jid,))
                credentials = res.fetchone()

                if credentials:
                    return Signal.RESET, SE.conflict_error(
                        element.attrib["id"])
                else:
                    pwd = query.find(
                        CN.clarkFromTuple(
                            (self._ns, "password"))).text
                    hash_pwd = hashlib.sha256(pwd.encode()).hexdigest()

                    con.execute(
                        "INSERT INTO credentials(jid, hash_pwd) VALUES (?, ?)", (new_jid, hash_pwd))
                    con.commit()
                    return Signal.RESET, self.iq_register_result(
                        element.attrib["id"])

        else:
            raise Exception()

    def handleAuth(
            self, element: ET.Element) -> Union[Tuple[Signal, bytes], bytes]:
        data = base64.b64decode(element.text).split("\x00".encode())
        jid = data[1].decode()
        keyhash = hashlib.sha256(data[2]).hexdigest()

        with closing(self._db_connection_factory()) as con:
            res = con.execute(
                "SELECT hash_pwd FROM credentials WHERE jid = ?", (jid,))
            hash_pwd = res.fetchone()

        if hash_pwd and hash_pwd[0] == keyhash:
            self._connection_manager.set_jid(self._peername, jid)
            return Signal.RESET, b"<success xmlns='urn:ietf:params:xml:ns:xmpp-sasl'/>"

        return SE.not_authorized()

    def iq_register_result(self, id: str) -> bytes:
        iq = ET.Element(
            "iq",
            attrib={
                "type": "result",
                "id": id,
                "from": "localhost"})
        return ET.tostring(iq)


def SASLFeature(
        mechanismList: List[mechanismEnum] = [
            mechanismEnum.PLAIN
        ]):
    element = ET.Element(
        "mechanisms",
        attrib={
            "xmlns": "urn:ietf:params:xml:ns:xmpp-sasl"
        }
    )

    for m in mechanismList:
        mechanism = ET.SubElement(element, "mechanism")
        mechanism.text = m.value

    return element



import pytest
from unittest.mock import MagicMock
from xml.etree.ElementTree import Element
from pyjabber.plugins.PluginManager import PluginManager
from pyjabber.plugins.roster.Roster import Roster
from pyjabber.plugins.xep_0199.xep_0199 import Ping
from pyjabber.stanzas.error import StanzaError as SE

def test_plugin_manager_initialization():
    manager = PluginManager("example@domain.com")
    assert 'jabber:iq:roster' in manager._plugins
    assert 'urn:xmpp:ping' in manager._plugins
    assert isinstance(manager._plugins['jabber:iq:roster'], type)
    assert isinstance(manager._plugins['urn:xmpp:ping'], type)


def test_feed_with_known_plugin():
    manager = PluginManager("example@domain.com")
    manager._activePlugins = {
        'jabber:iq:roster': MagicMock(spec=Roster),
        'urn:xmpp:ping': MagicMock(spec=Ping)
    }

    # Crear un elemento de prueba para el plugin Roster
    element = Element('iq', {'type': 'get'})
    subelement = Element('{jabber:iq:roster}query')
    element.append(subelement)

    manager.feed(element)
    assert manager._activePlugins['jabber:iq:roster'].feed.called


def test_feed_with_unknown_plugin():
    manager = PluginManager("example@domain.com")
    # Crear un elemento de prueba para un plugin no registrado
    element = Element('iq', {'type': 'get'})
    subelement = Element('{jabber:iq:unknown}query')
    element.append(subelement)

    result = manager.feed(element)
    assert result == SE.service_unavaliable()


def test_feed_with_no_child():
    manager = PluginManager("example@domain.com")
    element = Element('iq', {'type': 'result'})

    # No debería llamar a ningún plugin ni lanzar excepciones
    result = manager.feed(element)
    assert result is None


def test_feed_with_error_handling():
    manager = PluginManager("example@domain.com")
    manager._plugins = {
        'jabber:iq:roster': MagicMock(Roster)
    }
    # Simular que el plugin necesario no está activo ni disponible
    element = Element('iq', {'type': 'get'})
    subelement = Element('{urn:xmpp:ping}ping')
    element.append(subelement)

    result = manager.feed(element)
    assert result == SE.service_unavaliable()
    assert 'urn:xmpp:ping' not in manager._activePlugins




import pytest
from pyjabber.utils.ClarkNotation import deglose, clarkFromTuple, isClark
def test_deglose():
    assert deglose("{namespace}tag") == ("namespace", "tag")
    assert deglose("{http://example.com}element") == ("http://example.com", "element")
    assert deglose("{urn:ietf:params:xml:ns:xmpp-stanzas}message") == ("urn:ietf:params:xml:ns:xmpp-stanzas", "message")
    assert deglose("{}empty") == ("", "empty")

def test_clarkFromTuple():
    assert clarkFromTuple(("namespace", "tag")) == "{namespace}tag"
    assert clarkFromTuple(("http://example.com", "element")) == "{http://example.com}element"
    assert clarkFromTuple((None, "tag")) == "tag"

def test_isClark():
    assert isClark("{namespace}tag") is True
    assert isClark("{http://example.com}element") is True
    assert isClark("tag") is False
    assert isClark("{namespace} tag") is False
    assert isClark("{namespace}") is False

if __name__ == "__main__":
    pytest.main()



from xml.etree import ElementTree as ET
from pyjabber.plugins.xep_0199.xep_0199 import Ping

def test_ping_feed_happy_path():
    ping = Ping(jid="user@domain.com")
    element = ET.Element("iq", attrib={"to": "localhost", "id": "1234"})

    result = ping.feed(element)

    expected_result = ET.tostring(
        ET.Element(
            "iq",
            attrib={
                "from": "localhost",
                "id": "1234",
                "to": "localhost",
                "type": "result",
            },
        )
    )

    assert result == [expected_result]

def test_ping_feed_wrong_to_value():
    ping = Ping(jid="user@domain.com")
    element = ET.Element("iq", attrib={"to": "remotehost", "id": "1234"})

    result = ping.feed(element)

    assert result is None

def test_ping_feed_empty_element():
    ping = Ping(jid="user@domain.com")
    element = ET.Element("iq")

    result = ping.feed(element)

    assert result is None

def test_ping_feed_with_invalid_xml():
    ping = Ping(jid="user@domain.com")

    # Simulando una cadena XML inválida
    invalid_xml_string = "<iq to='localhost' id='1234'><invalid<xml></iq>"

    try:
        element = ET.fromstring(invalid_xml_string)
        result = ping.feed(element)
        assert result is None
    except ET.ParseError:
        pass

def test_ping_feed_with_additional_attributes():
    ping = Ping(jid="user@domain.com")
    element = ET.Element("iq", attrib={"to": "localhost", "id": "1234", "extra": "value"})

    result = ping.feed(element)

    expected_result = ET.tostring(
        ET.Element(
            "iq",
            attrib={
                "from": "localhost",
                "id": "1234",
                "to": "localhost",
                "type": "result",
            },
        )
    )

    assert result == [expected_result]



import xml.etree.ElementTree as ET
from contextlib import closing
from typing import List

from pyjabber.db.database import connection


def retrieve_roster(jid: str) -> List[str]:  # pragma: no cover
    with closing(connection()) as con:
        res = con.execute("SELECT * FROM roster WHERE jid = ?", (jid,))
        roster = res.fetchall()
    return roster


def update(id: int, item: ET) -> str:  # pragma: no cover
    with closing(connection()) as con:
        con.execute("UPDATE roster SET rosterItem = ? WHERE id = ?",
                    (ET.tostring(item).decode(), id))
        con.commit()
        res = con.execute("SELECT rosterItem from roster WHERE id = ?", (id,))
        res = res.fetchone()
    return res[0]



import socket

import pytest
from unittest.mock import Mock, MagicMock, patch
from xml.etree import ElementTree as ET
from base64 import b64encode
from hashlib import sha256
from uuid import uuid4
from pyjabber.utils import ClarkNotation as CN
from pyjabber.stream.StreamHandler import Stage, Signal, StreamHandler

def test_stage_enum():
    assert Stage.CONNECTED.value == 0
    assert Stage.OPENED.value == 1
    assert Stage.SSL.value == 2
    assert Stage.SASL.value == 3
    assert Stage.AUTH.value == 4
    assert Stage.BIND.value == 5
    assert Stage.READY.value == 6

def test_signal_enum():
    assert Signal.RESET.value == 0
    assert Signal.DONE.value == 1

def test_stream_handler_initialization():
    buffer = Mock()
    starttls = Mock()
    connection_manager = Mock()
    host = Mock()

    handler = StreamHandler(host, buffer, starttls, connection_manager)

    assert handler._buffer == buffer
    assert handler._starttls == starttls
    assert handler._connection_manager == connection_manager
    assert handler._stage == Stage.CONNECTED
    assert handler._elem is None
    assert handler._jid is None

def test_stream_handler_buffer_property():
    buffer = Mock()
    starttls = Mock()
    connection_manager = Mock()
    host = Mock()

    handler = StreamHandler(host, buffer, starttls, connection_manager)

    new_buffer = Mock()
    handler.buffer = new_buffer

    assert handler.buffer == new_buffer

def test_handle_open_stream_connected():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    host = Mock()

    handler = StreamHandler(host, buffer, starttls, connection_manager)

    handler.handle_open_stream()

    assert handler._stage == Stage.OPENED
    buffer.write.assert_called_once()

def test_handle_open_stream_opened():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    host = Mock()

    handler = StreamHandler(host, buffer, starttls, connection_manager)
    handler._stage = Stage.OPENED

    elem = ET.Element("starttls")
    result = handler.handle_open_stream(elem)

    assert handler._stage == Stage.SSL
    assert result == Signal.RESET
    buffer.write.assert_called_once()
    starttls.assert_called_once()

def test_handle_open_stream_ssl():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    host = Mock()

    handler = StreamHandler(host, buffer, starttls, connection_manager)
    handler._stage = Stage.SSL

    handler.handle_open_stream()

    assert handler._stage == Stage.SASL
    buffer.write.assert_called_once()

from pyjabber.features.SASLFeature import SASL, connection


@patch('pyjabber.features.SASLFeature.connection')
@patch('pyjabber.stream.StreamHandler.SASL')
def test_handle_open_stream_sasl_continue(mock_sasl, mock_connection):
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    host = Mock()

    handler = StreamHandler(host, buffer, starttls, connection_manager)
    handler._stage = Stage.SASL

    password = b'password'
    keyhash = sha256(password).hexdigest()

    mock_conn = MagicMock()
    mock_cursor = MagicMock()

    mock_cursor.fetchone.return_value = (keyhash,)
    mock_conn.execute.return_value = mock_cursor
    mock_connection.return_value = mock_conn

    elem = ET.Element("{urn:ietf:params:xml:ns:xmpp-sasl}auth")
    auth_text = b64encode(b'\x00username\x00password').decode('ascii')
    elem.text = auth_text

    # Ajustar la función _db_connection_factory para que tome un argumento
    def mock_db_connection_factory():
        return mock_conn

    sasl_instance = SASL(connection_manager=mock_connection, db_connection_factory=mock_db_connection_factory)
    sasl_instance._connections = MagicMock()
    mock_sasl.return_value = sasl_instance

    handler.handle_open_stream(elem)

    assert handler._stage == Stage.AUTH
    expected_response = b"<success xmlns='urn:ietf:params:xml:ns:xmpp-sasl'/>"
    buffer.write.assert_called_once_with(expected_response)
def test_handle_open_stream_auth():
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    host = Mock()

    handler = StreamHandler(host, buffer, starttls, connection_manager)
    handler._stage = Stage.AUTH

    handler.handle_open_stream()

    assert handler._stage == Stage.BIND
    buffer.write.assert_called_once()

def test_handle_open_stream_bind(monkeypatch):
    buffer = MagicMock()
    starttls = Mock()
    connection_manager = Mock()
    host = Mock()

    handler = StreamHandler(host, buffer, starttls, connection_manager)
    handler._stage = Stage.BIND

    iq_elem = ET.Element("iq", attrib={"type": "set", "id": "123"})
    bind_elem = ET.SubElement(iq_elem, CN.clarkFromTuple(("urn:ietf:params:xml:ns:xmpp-bind", "bind")))
    res_elem = ET.SubElement(bind_elem, CN.clarkFromTuple(("urn:ietf:params:xml:ns:xmpp-bind", "resource")))
    res_elem.text = "resource_id"

    connections = Mock()
    handler._connection_manager = connections
    connections.get_jid.return_value = "user"

    clark_notation_bind = CN.clarkFromTuple(("urn:ietf:params:xml:ns:xmpp-bind", "bind"))
    bind_elem_found = iq_elem.find(clark_notation_bind)
    assert bind_elem_found is not None, f"bind_elem_found should not be None, got {bind_elem_found}"

    result = handler.handle_open_stream(iq_elem)

    assert result == Signal.DONE
    buffer.write.assert_called_once()

    connections.get_jid.assert_called_once_with(buffer.get_extra_info('peername'))
    connections.set_jid.assert_called_once_with(
        buffer.get_extra_info('peername'),
        f'user@{host}/resource_id',
        buffer
    )

# Ejecutar los tests con pytest
if __name__ == "__main__":
    pytest.main()



import asyncio
import colorama
import os
import signal
import socket
import urllib.request
import wget

from contextlib import closing
from loguru import logger

from pyjabber.db.database import connection
from pyjabber.network.XMLProtocol import XMLProtocol
from pyjabber.network.server.incoming.XMLServerIncomingProtocol import XMLServerIncomingProtocol
from pyjabber.network.server.outcoming.XMLServerOutcomingProtocol import XMLServerOutcomingProtocol
from pyjabber.network.ConnectionManager import ConnectionManager
from pyjabber.stream.QueueMessage import QueueMessage
from pyjabber.webpage.adminPage import admin_instance
from pyjabber.network import CertGenerator

SERVER_FILE_PATH = os.path.dirname(os.path.abspath(__file__))


class Server:
    """
        Server class

        :param host: Host for the clients connections
        :param client_port: Port for client connections (5222 by default)
        :param server_port: Port for server-to-server connections (5269 by default)
        :param family: Type of AddressFamily (IPv4 or IPv6)
        :param connection_timeout: Max time without any response from a client. After that, the server will terminate the connection
        :param enable_tls1_3: Boolean. Enables the use of TLSv1.3 in the STARTTLS process
        :parm cert_path: Path to custom domain certs. By default, the server generates its own certificates for hostname
    """
    def __init__(
        self,
        host=socket.gethostname(),
        client_port=5222,
        server_port=5269,
        family=socket.AF_INET,
        connection_timeout=60,
        enable_tls1_3=False,
        cert_path=None
    ):
        # Server
        self._host = host
        self._client_port = client_port
        self._server_port = server_port
        self._family = family
        self._client_listener = None
        self._server_listener = None
        self._adminServer = None
        self._public_ip = None
        self._connection_timeout = connection_timeout
        self._cert_path = cert_path

        # Client handler
        self._enable_tls1_3 = enable_tls1_3
        self._connection_manager = ConnectionManager(self.task_s2s)
        self._queue_message = QueueMessage(self._connection_manager)

    async def run_server(self):
        logger.info("Starting server...")

        if os.path.isfile(os.path.join(SERVER_FILE_PATH, "db", "server.db")) is False:
            logger.debug("No database found. Initializing one...")
            with closing(connection()) as con:
                with open(SERVER_FILE_PATH + "/db/schema.sql", "r") as schema:
                    con.cursor().executescript(schema.read())
                con.commit()

        if CertGenerator.check_hostname_cert_exists(self._host) is False:
            CertGenerator.generate_hostname_cert(self._host)

        if not os.path.isfile(os.path.join(SERVER_FILE_PATH, "network", "certs", "traefik.pem")):
            wget.download("http://traefik.me/fullchain.pem", os.path.join(SERVER_FILE_PATH, "network", "certs", "traefik.pem"))
        if not os.path.isfile(os.path.join(SERVER_FILE_PATH, "network", "certs", "traefik-key.pem")):
            wget.download("http://traefik.me/privkey.pem", os.path.join(SERVER_FILE_PATH, "network", "certs", "traefik-key.pem"))

        loop = asyncio.get_running_loop()

        lan_ip = self.query_local_ip()

        self._client_listener = await loop.create_server(
            lambda: XMLProtocol(
                namespace='jabber:client',
                host=self._host,
                connection_timeout=self._connection_timeout,
                connection_manager=self._connection_manager,
                cert_path=self._cert_path,
                queue_message=self._queue_message,
                enable_tls1_3=self._enable_tls1_3,
            ),
            host=[self._host, lan_ip],
            port=self._client_port,
            family=self._family
        )

        logger.info(f"Client domain => {self._host}")
        logger.info(f"Server is listening clients on {[s.getsockname() for s in self._client_listener.sockets]}")

        self._server_listener = await loop.create_server(
            lambda: XMLServerIncomingProtocol(
                namespace='jabber:server',
                host=self._host,
                connection_timeout=self._connection_timeout,
                connection_manager=self._connection_manager,
                cert_path=self._cert_path,
                queue_message=self._queue_message,
                enable_tls1_3=self._enable_tls1_3,
            ),
            host="0.0.0.0",
            port=self._server_port,
            family=self._family
        )

        logger.info(f"Server is listening servers on {[s.getsockname() for s in self._server_listener.sockets]}")

        public_ip = urllib.request.urlopen("https://api.ipify.org/")
        if public_ip.status == 200:
            self._public_ip = public_ip.read().decode()
            self._public_ip = self._public_ip.replace(".", "-") + ".traefik.me"
            logger.info(f"S2S domain ==> {colorama.Fore.RED}{self._public_ip.replace('.', '-')}")

        logger.info("Server started...")

    async def stop(self):
        logger.info("Stopping server...")

        if self._client_listener and self._client_listener.is_serving():
            self._client_listener.close()
            await self._client_listener.wait_closed()

        if self._server_listener and self._server_listener.is_serving():
            self._server_listener.close()
            await self._server_listener.wait_closed()

        logger.info("Server stopped...")

    def task_s2s(self, host):
        host = host.split("@")[-1]
        loop = asyncio.get_running_loop()

        asyncio.ensure_future(self.server_connection(host, loop), loop=loop)

    async def server_connection(self, remote_host, loop):
        """
            Task to connect to another XMPP server
            :param remote_host: Host of the remote server to connect
            :param loop: Asyncio running loop. Necessary to perform task
        """
        await loop.create_connection(
            lambda: XMLServerOutcomingProtocol(
                namespace="jabber:server",
                host=remote_host,
                public_host=self._public_ip,
                connection_timeout=self._connection_timeout,
                connection_manager=self._connection_manager,
                queue_message=self._queue_message,
                enable_tls1_3=self._enable_tls1_3,
            ),
            host=remote_host,
            port=5269
        )

    def query_local_ip(self):
        """
            Return the local IP of the host machine
        """
        try:
            connection = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            connection.connect(("8.8.8.8", 80))
            local_ip_address = connection.getsockname()[0]
            return local_ip_address
        except Exception as e:
            logger.error("Unable to retrieve LAN IP")
            raise self.raise_exit()
        finally:
            connection.close()

    def raise_exit(self):
        raise SystemExit(1)

    def start(self, debug: bool = False):
        """
            Start the already created and configuration server
            :param debug: Boolean. Enables debug mode in asyncio
        """
        loop = asyncio.get_event_loop()
        loop.set_debug(debug)

        try:
            loop.add_signal_handler(signal.SIGINT, self.raise_exit)
            loop.add_signal_handler(signal.SIGABRT, self.raise_exit)
            loop.add_signal_handler(signal.SIGTERM, self.raise_exit)
        except NotImplementedError:  # pragma: no cover
            pass

        try:
            # XMPP Server
            main_server = loop.create_task(self.run_server())
            loop.run_until_complete(main_server)

            # Control Panel Webpage | localhost:9090
            admin_server = admin_instance()
            loop.run_until_complete(admin_server)

        except (SystemExit, KeyboardInterrupt):  # pragma: no cover
            pass

        finally:
            # Cancel pending tasks
            tasks = asyncio.all_tasks(loop)
            for task in tasks:
                task.cancel()
            loop.run_until_complete(asyncio.gather(*tasks, return_exceptions=True))

            # Close the server
            close_task = loop.create_task(self.stop())
            loop.run_until_complete(close_task)
            loop.close()
            asyncio.set_event_loop(None)



import xml.etree.ElementTree as ET
from typing import Dict

from pyjabber.plugins.PluginInterface import Plugin

# Plugins
from pyjabber.plugins.roster.Roster import Roster
from pyjabber.plugins.xep_0199.xep_0199 import Ping
from pyjabber.stanzas.error import StanzaError as SE
from pyjabber.utils import ClarkNotation as CN


class PluginManager():
    def __init__(self, jid) -> None:

        self._jid = jid
        self._plugins: Dict[str, Plugin] = {
            'jabber:iq:roster': Roster,
            'urn:xmpp:ping': Ping
        }
        self._activePlugins: Dict[str, Plugin] = {}

    def feed(self, element: ET.Element):
        try:
            child = element[0]
        except IndexError:
            if element.attrib["type"] == "result":
                return
            else:
                return SE.service_unavaliable()

        tag, _ = CN.deglose(child.tag)

        try:
            plugin = self._activePlugins[tag]  # Plugin already instanced
            return plugin.feed(element)
        except KeyError:
            try:
                # Retrive plugin from list and instance
                plugin = self._plugins[tag]
                self._activePlugins[tag] = plugin(self._jid)
                return self._activePlugins[tag].feed(element)
            except KeyError:
                return SE.service_unavaliable()  # Plugin unavailable



import asyncio
from unittest.mock import MagicMock, patch, AsyncMock
import pytest
from xml import sax
import ssl
from pyjabber.network.XMLProtocol import XMLProtocol, FILE_AUTH
import os


@patch('pyjabber.network.XMLProtocol.sax.make_parser')
@patch('pyjabber.network.XMLProtocol.ConnectionManager')
def test_connection_made(mock_connections_manager, mock_make_parser):
    mock_transport = MagicMock()
    mock_parser = MagicMock()
    mock_make_parser.return_value = mock_parser
    mock_handler = MagicMock()

    namespace = "jabber:client"
    host = "domain.com"
    cert_path = None
    connection_timeout = 30
    connection_manager = mock_connections_manager.return_value
    queue_message = MagicMock()

    with patch('pyjabber.network.XMLProtocol.XMLParser', return_value=mock_handler):
        protocol = XMLProtocol(
            host=host,
            namespace=namespace,
            connection_timeout=connection_timeout,
            connection_manager=connection_manager,
            cert_path=cert_path,
            queue_message=queue_message
        )
        protocol.connection_made(mock_transport)

    mock_transport.get_extra_info.assert_called_with('peername')
    mock_make_parser.assert_called_once()
    mock_parser.setFeature.assert_any_call(sax.handler.feature_namespaces, True)
    mock_parser.setFeature.assert_any_call(sax.handler.feature_external_ges, False)
    mock_parser.setContentHandler.assert_called_once_with(mock_handler)
    mock_connections_manager.return_value.connection.assert_called_once_with(mock_transport.get_extra_info.return_value)


@patch('pyjabber.network.XMLProtocol.StreamAlivenessMonitor')
def test_connection_made_with_timeout(mock_monitor):
    mock_transport = MagicMock()
    mock_parser = MagicMock()

    namespace = "jabber:client"
    host = MagicMock()
    cert_path = None
    connection_timeout = 30
    connection_manager = MagicMock()
    queue_message = MagicMock()

    with patch('pyjabber.network.XMLProtocol.sax.make_parser', return_value=mock_parser):
        protocol = XMLProtocol(
            host=host,
            namespace=namespace,
            cert_path=cert_path,
            connection_timeout=connection_timeout,
            connection_manager=connection_manager,
            queue_message=queue_message
        )
        protocol.connection_made(mock_transport)

    mock_monitor.assert_called_once_with(timeout=30, callback=protocol.connection_timeout)
    mock_transport.get_extra_info.assert_called_with('peername')


def test_connection_lost():
    namespace = "jabber:client"
    host = MagicMock()
    cert_path = None
    connection_timeout = 30
    connection_manager = MagicMock()
    queue_message = MagicMock()

    protocol = XMLProtocol(
        host=host,
        namespace=namespace,
        connection_timeout=connection_timeout,
        connection_manager=connection_manager,
        cert_path=cert_path,
        queue_message=queue_message
    )
    mock_transport = MagicMock()
    protocol._transport = mock_transport

    protocol.connection_lost(None)
    mock_transport.get_extra_info.assert_called_with('peername')
    assert protocol._transport is None
    assert protocol._xml_parser is None


def test_data_received():
    namespace = "jabber:client"
    host = MagicMock()
    cert_path = None
    connection_timeout = 30
    connection_manager = MagicMock()
    queue_message = MagicMock()

    protocol = XMLProtocol(
        host=host,
        namespace=namespace,
        connection_timeout=connection_timeout,
        connection_manager=connection_manager,
        cert_path=cert_path,
        queue_message=queue_message
    )
    mock_transport = MagicMock()
    protocol._transport = mock_transport
    mock_parser = MagicMock()
    protocol._xml_parser = mock_parser
    mock_monitor = MagicMock()
    protocol._timeout_monitor = mock_monitor

    data = b"<?xml version=\'1.0\'?><stream:stream>"
    protocol.data_received(data)
    mock_monitor.reset.assert_called_once()
    mock_parser.feed.assert_called_once_with(b"<stream:stream>")


def test_eof_received():
    namespace = "jabber:client"
    host = MagicMock()
    cert_path = None
    connection_timeout = 30
    connection_manager = MagicMock()
    queue_message = MagicMock()

    protocol = XMLProtocol(
        host=host,
        namespace=namespace,
        connection_timeout=connection_timeout,
        connection_manager=connection_manager,
        cert_path=cert_path,
        queue_message=queue_message
    )
    mock_transport = MagicMock()
    protocol.connection_made(mock_transport)
    mock_connections = MagicMock()
    protocol._connection_manager = mock_connections

    protocol.eof_received()
    mock_transport.get_extra_info.assert_called_with('peername')
    mock_connections.disconnection.assert_called_once_with(mock_transport.get_extra_info.return_value)

    # Verificar que el transporte y el analizador XML se mantengan en su estado actual
    assert protocol._transport == mock_transport
    assert protocol._xml_parser is not None  # El parser debe estar inicializado en connection_made



def test_connection_timeout():
    namespace = "jabber:client"
    host = MagicMock()
    cert_path = None
    connection_timeout = 30
    connection_manager = MagicMock()
    queue_message = MagicMock()

    protocol = XMLProtocol(
        host=host,
        namespace=namespace,
        connection_timeout=connection_timeout,
        connection_manager=connection_manager,
        cert_path=cert_path,
        queue_message=queue_message
    )
    mock_transport = MagicMock()
    protocol._transport = mock_transport

    protocol.connection_timeout()
    mock_transport.get_extra_info.assert_called_with('peername')
    mock_transport.write.assert_called_once_with(b"<connection-timeout/>")
    mock_transport.close.assert_called_once()
    assert protocol._transport is None
    assert protocol._xml_parser is None


@patch('pyjabber.network.XMLProtocol.asyncio.get_running_loop')
@patch('pyjabber.network.XMLProtocol.ssl.create_default_context')
def test_enable_tls(mock_create_default_context, mock_get_running_loop):
    namespace = "jabber:client"
    host = MagicMock()
    cert_path = None
    connection_timeout = 30
    connection_manager = MagicMock()
    queue_message = MagicMock()

    protocol = XMLProtocol(
        host=host,
        namespace=namespace,
        connection_timeout=connection_timeout,
        connection_manager=connection_manager,
        cert_path=cert_path,
        queue_message=queue_message
    )
    mock_transport = MagicMock()
    protocol._transport = mock_transport
    mock_loop = MagicMock()
    mock_get_running_loop.return_value = mock_loop

    # Crear un contexto SSL real para evitar el error isinstance
    real_ssl_context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
    mock_create_default_context.return_value = real_ssl_context

    # Mockear el parser para evitar el error 'NoneType'
    mock_parser = MagicMock()
    protocol._xml_parser = mock_parser

    # Asegurarse de que start_tls es un AsyncMock
    mock_loop.start_tls = AsyncMock()

    async def run_enable_tls():
        await protocol.enable_tls()
        mock_create_default_context.assert_called_once_with(ssl.Purpose.CLIENT_AUTH)
        real_ssl_context.load_cert_chain.assert_called_once_with(
            certfile=os.path.join(FILE_AUTH, 'certs', 'localhost.pem'),
            keyfile=os.path.join(FILE_AUTH, 'certs', 'localhost-key.pem')
        )
        mock_loop.start_tls.assert_called_once_with(
            transport=mock_transport,
            protocol=protocol,
            sslcontext=real_ssl_context,
            server_side=True
        )

    mock_loop.run_until_complete(run_enable_tls())




import pytest
from unittest.mock import MagicMock
from pyjabber.network.ConnectionManager import ConnectionManager
from asyncio import Transport

from loguru import logger
import logging

@pytest.fixture
def connections_manager():
    task_s2s_mock = MagicMock()  # Creamos un mock para el argumento requerido
    return ConnectionManager(task_s2s=task_s2s_mock)
def test_connection(connections_manager):
    peer = ("127.0.0.1", 12345)
    connections_manager.connection(peer)
    assert peer in connections_manager._peerList
    assert connections_manager._peerList[peer] == {"jid": None, "transport": None}

def test_disconnection(connections_manager):
    peer = ("127.0.0.1", 12345)
    connections_manager.connection(peer)
    connections_manager.disconnection(peer)
    assert peer not in connections_manager._peerList

def test_get_users_connected(connections_manager):
    peer = ("127.0.0.1", 12345)
    connections_manager.connection(peer)
    users_connected = connections_manager.get_users_connected()
    assert users_connected == {peer: {"jid": None, "transport": None}}

def test_get_buffer(connections_manager):
    peer = ("127.0.0.1", 12345)
    transport = MagicMock(spec=Transport)
    connections_manager.connection(peer)
    connections_manager.set_jid(peer, "user@localhost", transport)
    buffer = connections_manager.get_buffer("user@localhost")
    assert buffer == [("user@localhost", transport)]

def test_get_jid(connections_manager):
    peer = ("127.0.0.1", 12345)
    connections_manager.connection(peer)
    connections_manager.set_jid(peer, "user@localhost")
    jid = connections_manager.get_jid(peer)
    assert jid == "user@localhost"

def test_set_jid(connections_manager):
    peer = ("127.0.0.1", 12345)
    transport = MagicMock(spec=Transport)
    connections_manager.connection(peer)
    connections_manager.set_jid(peer, "user@localhost", transport)
    assert connections_manager._peerList[peer] == {"jid": "user@localhost", "transport": transport}

def test_set_jid_without_transport(connections_manager):
    peer = ("127.0.0.1", 12345)
    connections_manager.connection(peer)
    connections_manager.set_jid(peer, "user@localhost")
    assert connections_manager._peerList[peer] == {"jid": "user@localhost", "transport": None}

def test_set_jid_key_error(connections_manager):
    peer = ("127.0.0.1", 12345)
    result = connections_manager.set_jid(peer, "user@localhost")
    assert result is False


@pytest.fixture(autouse=True)
def setup_logging(caplog):
    # Remover todos los handlers para evitar duplicados
    logger.remove()
    # Configurar loguru para trabajar con caplog
    logger.add(caplog.handler, level="ERROR")

def test_disconnection_key_error(connections_manager, caplog):
    peer = ("127.0.0.1", 12345)
    with caplog.at_level(logging.ERROR):
        connections_manager.disconnection(peer)
    assert peer not in connections_manager._peerList
    assert any("not present in the online list" in record.message for record in caplog.records)



def test_check_server_present_in_list_false(connections_manager):
    host = "server.localhost"
    assert not connections_manager.check_server_present_in_list(host)

def test_get_server_buffer_task_s2s(connections_manager):
    host = "server.localhost"
    connections_manager.get_server_buffer(host)
    connections_manager._task_s2s.assert_called_with(host)

@pytest.fixture(autouse=True)
def cleanup(connections_manager):
    yield
    connections_manager._peerList.clear()

def test_connection_server(connections_manager):
    peer = ("127.0.0.1", 12345)
    host = "server.localhost"
    transport = MagicMock(spec=Transport)
    connections_manager.connection_server(peer, host, transport)
    assert peer in connections_manager._remoteList
    assert connections_manager._remoteList[peer] == {"jid": host, "transport": None}

def test_disconnection_server(connections_manager):
    peer = ("127.0.0.1", 12345)
    host = "server.localhost"
    transport = MagicMock(spec=Transport)
    connections_manager.connection_server(peer, host, transport)
    connections_manager.disconnection_server(peer)
    assert peer not in connections_manager._remoteList

def test_get_server_buffer(connections_manager):
    peer = ("127.0.0.1", 12345)
    host = "server.localhost"
    transport = MagicMock(spec=Transport)
    connections_manager.connection_server(peer, host, transport)
    connections_manager.set_server_transport(peer, transport)
    buffer = connections_manager.get_server_buffer(host)
    assert buffer == (host, transport)

def test_set_server_transport(connections_manager):
    peer = ("127.0.0.1", 12345)
    host = "server.localhost"
    transport = MagicMock(spec=Transport)
    connections_manager.connection_server(peer, host, None)
    connections_manager.set_server_transport(peer, transport)
    assert connections_manager._remoteList[peer]["transport"] == transport

def test_check_server_stream_available(connections_manager):
    peer = ("127.0.0.1", 12345)
    host = "server.localhost"
    transport = MagicMock(spec=Transport)
    connections_manager.connection_server(peer, host, transport)
    connections_manager.set_server_transport(peer, transport)
    available = connections_manager.check_server_stream_available(host)
    assert available is True

def test_check_server_present_in_list(connections_manager):
    peer = ("127.0.0.1", 12345)
    host = "server.localhost"
    transport = MagicMock(spec=Transport)
    connections_manager.connection_server(peer, host, transport)
    present = connections_manager.check_server_present_in_list(host)
    assert present is True

def test_get_server_buffer_no_transport(connections_manager):
    peer = ("127.0.0.1", 12345)
    host = "server.localhost"
    connections_manager.connection_server(peer, host, None)
    connections_manager.set_server_transport(peer, None)
    buffer = connections_manager.get_server_buffer(host)
    assert buffer is None

def test_check_server_stream_available_false(connections_manager):
    peer = ("127.0.0.1", 12345)
    host = "server.localhost"
    connections_manager.connection_server(peer, host, None)
    available = connections_manager.check_server_stream_available(host)
    assert available is False





from typing import Dict
from xml.etree import ElementTree as ET


class InBandRegistration(ET.Element):
    def __init__(
        self,
        tag: str = "register",
        attrib: Dict[str, str] = {
            "xmlns": "http://jabber.org/features/iq-register"
        },
            **extra: str) -> None:
        super().__init__(tag, attrib, **extra)



from setuptools import setup, find_packages
import pyjabber

setup(
    name="pyjabber",
    version=pyjabber.__version__,
    author="Aarón Raya  ",
    author_email="aaron.raya.lopez@gmail.com",
    description="A Python XMPP server",
    long_description=open("README.rst").read(),
    long_description_content_type="text/x-rst",
    url="https://github.com/DinoThor/PyJabber",
    packages=find_packages(),
    install_requires=[
        "aiohttp==3.9.5",
        "click==8.1.7",
        "loguru==0.7.2",
        "nest-asyncio==1.6.0",
        "setuptools~=69.5.1",
        "slixmpp~=1.8.5",
        "wget~=3.2",
        "xmlschema==3.3.0",
        "cryptography~=42.0.8",
        "colorama~=0.4.6"
    ],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
    entry_points={
        "console_scripts": [
            "pyjabber=pyjabber.__main__:main",
        ],
    },
)



# https://sabre.io/xml/clark-notation/
import re


def deglose(tag: str):
    namespace = tag.split("}")[0].replace("{", "")
    tag = tag.split("}")[1]
    return namespace, tag


def clarkFromTuple(tuple):
    if tuple[0] is None:
        return f"{tuple[1]}"
    return f"{{{tuple[0]}}}{tuple[1]}"


def isClark(tag: str) -> bool:
    return re.match(
        "^\\{[a-zA-Z0-9:/#?=&;.]+?\\}[a-zA-Z_][a-zA-Z0-9_]*$",
        tag) is not None



import os
from unittest.mock import patch, mock_open, MagicMock
import pickle
import xmlschema


def load_schemas():
    SERVER_FILE_PATH = os.path.dirname(os.path.abspath(__file__))

    schemas = {
        'http://jabber.org/protocol/activity': "https://xmpp.org/schemas/activity.xsd",
        'jabber:iq:register': "https://xmpp.org/schemas/iq-register.xsd",
        'jabber:iq:roster': "https://xmpp.org/schemas/roster.xsd",
        'urn:ietf:params:xml:ns:xmpp-bind': "https://xmpp.org/schemas/bind.xsd",
        'jabber:client': "https://xmpp.org/schemas/jabber-client.xsd",
        'jabber:server': "https://xmpp.org/schemas/jabber-server.xsd"
    }

    if os.path.exists(SERVER_FILE_PATH + "/schemas.pkl") is False:
        res = {}
        for ns, url in schemas.items():
            res[ns] = xmlschema.XMLSchema(url)

        with open(SERVER_FILE_PATH + "/schemas.pkl", "wb") as file:
            pickle.dump(res, file)
    return res
@patch('os.path.exists')
@patch('builtins.open', new_callable=mock_open)
def test_file_creation(mock_open, mock_exists):
    # Simular que el archivo no existe
    mock_exists.return_value = False

    # Ejecutar la función a testear
    _ = load_schemas()

    # Verificar que el archivo se abre para escribir
    mock_open.assert_called_with((os.path.dirname(os.path.abspath(__file__)) + "/schemas.pkl"), "wb")



@patch('os.path.exists')
def test_schema_loading(mock_exists):
    mock_exists.return_value = False

    res = load_schemas()

    for ns, schema in res.items():
        assert isinstance(schema, xmlschema.XMLSchema)





import xml.etree.ElementTree as ET
from abc import ABCMeta, abstractmethod
from typing import Dict


class FeatureInterface(metaclass=ABCMeta):
    @abstractmethod
    def feed(self, element: ET.Element, extra: Dict[str, any] = None):
        pass



import os
import socket
import pytest
from unittest.mock import patch, MagicMock, mock_open
from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.x509.oid import NameOID
from pyjabber.network.CertGenerator import check_hostname_cert_exists, \
    generate_hostname_cert

CERTS_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), '../pyjabber/network/certs'))


@pytest.fixture
def hostname():
    return "example.com"


def test_check_hostname_cert_exists_missing_file(hostname):
    def mock_isfile(path):
        # Simula que los archivos de certificados no existen
        return False

    with patch('os.path.isfile', side_effect=mock_isfile), \
        patch('os.chdir') as mock_chdir, \
        patch('loguru.logger.debug') as mock_logger_debug:
        result = check_hostname_cert_exists(hostname)

        mock_chdir.assert_any_call(CERTS_PATH)
        mock_chdir.assert_any_call(os.getcwd())
        mock_logger_debug.assert_called_with("Missing hostname certificate")
        assert result is False


def test_check_hostname_cert_exists_all_files_present(hostname):
    with patch('os.path.isfile', return_value=True), \
        patch('os.chdir') as mock_chdir:
        result = check_hostname_cert_exists(hostname)

        mock_chdir.assert_any_call(CERTS_PATH)
        mock_chdir.assert_any_call(os.getcwd())
        assert result is True


@pytest.mark.parametrize("file_exists", [True, False])
def test_generate_hostname_cert(hostname, file_exists):
    # Crear una clave privada real
    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
        backend=default_backend()
    )
    # Crear una clave pública real
    public_key = private_key.public_key()

    # Crear mocks
    mock_csr = MagicMock()
    mock_ca_key = MagicMock()
    mock_ca_cert = MagicMock()
    mock_domain_cert = MagicMock()

    # Crear un objeto x509.Name para usar como subject e issuer
    subject_name = x509.Name([
        x509.NameAttribute(NameOID.ORGANIZATION_NAME, u"Spade"),
        x509.NameAttribute(NameOID.COMMON_NAME, hostname),
    ])

    # Ajustar los retornos de los mocks para que sean del tipo esperado
    mock_csr.subject = subject_name
    mock_csr.public_key.return_value = public_key
    mock_ca_cert.subject = subject_name

    with patch('os.path.isfile', return_value=file_exists), \
        patch('os.remove') as mock_remove, \
        patch('os.chdir') as mock_chdir, \
        patch('builtins.open', mock_open()), \
        patch('cryptography.hazmat.primitives.asymmetric.rsa.generate_private_key', return_value=private_key), \
        patch('cryptography.x509.CertificateSigningRequestBuilder.sign', return_value=mock_csr), \
        patch('cryptography.hazmat.primitives.serialization.load_pem_private_key', return_value=mock_ca_key), \
        patch('cryptography.x509.load_pem_x509_certificate', return_value=mock_ca_cert), \
        patch('cryptography.x509.CertificateBuilder.sign', return_value=mock_domain_cert), \
        patch('loguru.logger.debug') as mock_logger_debug, \
        patch('socket.gethostname', return_value=hostname):
        generate_hostname_cert(hostname)

        mock_chdir.assert_any_call(CERTS_PATH)
        mock_chdir.assert_any_call(os.getcwd())

        if file_exists:
            assert mock_remove.call_count == 3

        mock_logger_debug.assert_any_call("Generating hostname certificate")
        mock_logger_debug.assert_any_call(f"Certificate generated for => {hostname}")



import xml.etree.ElementTree as ET
from typing import Any, Dict
from uuid import uuid4
from xml.etree.ElementTree import Element

from pyjabber.features.feature_utils import RosterUtils as RU
from pyjabber.features.FeatureInterface import FeatureInterface
from pyjabber.features.presence.utils import create_roster_entry
from pyjabber.plugins.roster.Roster import Roster


class Presence(FeatureInterface):
    def __init__(self, jid, connection_manager) -> None:
        self._handlers = {
            "subscribe": self.handle_subscribe,
            "subscribed": self.handle_subscribed,
            "unsubscribed": self.handle_unsubscribed,
            "unavailable": self.handle_unavailable
        }
        self._connections = connection_manager
        self._jid = jid

    def feed(self, element: Element, extra: Dict[str, Any] = None):
        if "type" in element.attrib:
            return self._handlers[element.attrib["type"]](element)

        if "to" not in element.attrib:
            return self.handle_initial_presence(element)

        return None

    def handle_subscribe(self, element: ET.Element):
        to = element.attrib["to"].split("/")[0]
        bare_jid = self._jid.split("/")[0]
        roster_manager = Roster(bare_jid)

        if "from" not in element.attrib:
            element.attrib["from"] = self._jid

        # Handle presence locally
        if to.split("@")[1] == "localhost":
            roster = RU.retrieve_roster(bare_jid)
            buffer = self._connections.get_buffer(to)

            item = [item for item in roster
                    if ET.fromstring(item[2]).attrib["jid"] == to]

            if not item:
                create_roster_entry(bare_jid, to, roster_manager)

                roster = RU.retrieve_roster(bare_jid)
                buffer = self._connections.get_buffer(to)

                item = [item for item in roster
                        if ET.fromstring(item[2]).attrib["jid"] == to]

            item = item[0]
            ETitem = ET.fromstring(item[2])

            if ETitem.attrib["subscription"] in ["to", "both"]:
                petition = ET.Element(
                    "presence",
                    attrib={
                        "from": element.attrib['to'],
                        "to": element.attrib['from'],
                        "id": element.attrib['id'],
                        "type": "subscribed"
                    }
                )
                return ET.tostring(petition)

            if "ask" not in ETitem.attrib:
                newItem = ETitem.__copy__()
                newItem.attrib["ask"] = "subscribe"
                RU.update(item=newItem, id=item[0])

            petition = ET.Element(
                "presence",
                attrib={
                    "from": element.attrib['from'],
                    "to": element.attrib['to'],
                    "id": element.attrib['id'],
                    "type": "subscribe"
                }
            )
            for b in buffer:
                b[-1].write(ET.tostring(petition))
                return

    def handle_subscribed(self, element: ET.Element):
        to = element.attrib["to"]
        bare_jid = self._jid.split("/")[0]

        if "from" not in element.attrib:
            element.attrib["from"] = self._jid

        if to.split("@")[1] == "localhost":
            bufferBob = self._connections.get_buffer(to)
            bufferAlice = self._connections.get_buffer(bare_jid)

            rosterBob = RU.retrieve_roster(to)
            rosterAlice = RU.retrieve_roster(bare_jid)

            if not rosterAlice:
                create_roster_entry(bare_jid, to, Roster(bare_jid))
                rosterAlice = RU.retrieve_roster(bare_jid)

            rosterPushBob = None
            rosterPushAlice = None

            itemBob = [  # Alice appears in Bob's roster
                item
                for item in rosterBob
                if ET.fromstring(item[2]).attrib["jid"] == bare_jid
            ]

            itemAlice = [  # Bob appears in Alice's roster
                item
                for item in rosterAlice
                if ET.fromstring(item[2]).attrib["jid"] == to
            ]

            if itemBob:
                itemBob = itemBob[0]
                ETitemBob = ET.fromstring(itemBob[2])
                newItemBob = ETitemBob.__copy__()

                if ETitemBob.attrib["subscription"] == "none":
                    if "ask" in newItemBob.attrib:
                        newItemBob.attrib.pop("ask")
                        newItemBob.attrib["subscription"] = "to"

                if ETitemBob.attrib["subscription"] == "from":
                    if "ask" in newItemBob.attrib:
                        newItemBob.attrib.pop("ask")
                        newItemBob.attrib["subscription"] = "both"

                rosterPushBob = RU.update(item=newItemBob, id=itemBob[0])

            if itemAlice:
                itemAlice = itemAlice[0]
                ETitemAlice = ET.fromstring(itemAlice[2])
                newItemAlice = ETitemAlice.__copy__()

                if ETitemAlice.attrib["subscription"] == "none":
                    newItemAlice.attrib["subscription"] = "from"

                if ETitemAlice.attrib["subscription"] == "to":
                    newItemAlice.attrib["subscription"] = "both"

                rosterPushAlice = RU.update(item=newItemAlice, id=itemAlice[0])

            for bob in bufferBob:
                res = ET.Element(
                    "presence",
                    attrib={
                        "from": bare_jid,
                        "to": to,
                        "id": element.attrib["id"],
                        "type": "subscribed",
                    },
                )

                bob[-1].write(ET.tostring(res))

            if rosterPushBob:
                for bob in bufferBob:
                    res = ET.Element(
                        "iq",
                        attrib={
                            "id": str(uuid4()),
                            "to": bob[0],
                            "type": "set"
                        }
                    )
                    query = ET.Element(
                        "query", attrib={
                            "xmlns": "jabber:iq:roster"})
                    item = ET.fromstring(rosterPushBob)

                    query.append(item)
                    res.append(query)

                    bob[-1].write(ET.tostring(res))

            if rosterPushAlice:
                for alice in bufferAlice:
                    res = ET.Element(
                        "iq",
                        attrib={
                            "id": str(uuid4()),
                            "to": alice[0],
                            "type": "set"
                        }
                    )
                    query = ET.Element(
                        "query", attrib={
                            "xmlns": "jabber:iq:roster"})
                    item = ET.fromstring(rosterPushAlice)

                    query.append(item)
                    res.append(query)

                    bob[-1].write(ET.tostring(res))

    def handle_initial_presence(self, element: ET.Element):
        bare_jid = self._jid.split("/")[0]
        roster = RU.retrieve_roster(bare_jid)

        for r in roster:
            item = ET.fromstring(r[-1])
            jid = item.attrib["jid"]
            buffer = self._connections.get_buffer(jid)
            for b in buffer:
                presence = ET.Element(
                    "presence",
                    attrib={
                        "from": self._jid,
                        "to": b[0].split("/")[0]})
                b[-1].write(ET.tostring(presence))

    def handle_unsubscribed(self, element: ET.Element):
        to = element.attrib["to"].split("/")[0]
        bare_jid = self._jid.split("/")[0]

        if "from" not in element.attrib:
            element.attrib["from"] = self._jid

        # Handle locally
        if to.split("@")[1] == "localhost":
            roster = RU.retrieve_roster(bare_jid)
            buffer = self._connections.get_buffer(to)

            item = [item for item in roster
                    if ET.fromstring(item[2]).attrib["jid"] == to]

            if not item:
                return

            item = item[0]
            ETitem = ET.fromstring(item[2])

            newItem = ETitem.__copy__()

            if ETitem.attrib["subscription"] == "to":
                newItem.attrib["subscription"] = "none"
                RU.update(item=newItem, id=item[0])

            elif ETitem.attrib["subscription"] == "both":
                newItem.attrib["subscription"] = "from"
                RU.update(item=newItem, id=item[0])

            elif "ask" in ETitem.attrib:
                newItem.attrib.pop("ask")
                RU.update(item=newItem, id=item[0])

            id = str(uuid4())

            for b in buffer:
                presence = ET.Element(
                    "presence",
                    attrib={
                        "to": b[0],
                        "from": element.attrib['from'],
                        "id": element.attrib['id'],
                        "type": "unsubscribed"
                    }
                )

                roster_push = ET.Element(
                    "iq",
                    attrib={
                        "id": id,
                        "to": b[0],
                        "type": "set"
                    }
                )

                query = ET.Element("{jabber:iq:roster}query")
                query.append(newItem)
                roster_push.append(query)

                b[-1].write(ET.tostring(presence))
                b[-1].write(ET.tostring(roster_push))

    def handle_unavailable(self, element: ET.Element):
        bare_jid = self._jid.split("/")[0]

        if "from" not in element.attrib:
            element.attrib["from"] = self._jid

        roster = RU.retrieve_roster(bare_jid)

        for item in roster:
            to = ET.fromstring(item[-1]).attrib["jid"]

            if to.split("@")[1] == "localhost":
                presence = element.__copy__()

                presence.attrib["to"] = to

                buffer = self._connections.get_buffer(to)
                for b in buffer:
                    b[-1].write(ET.tostring(presence))
